# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Pegasus(KaitaiStruct):
    """:field pid: tt64_frame.pid
    :field call: tt64_frame.beacon.call
    :field trx_temp: tt64_frame.beacon.trx_temp
    :field antenna_deployment: tt64_frame.beacon.antenna_deployment
    :field stacie_op: tt64_frame.beacon.stacie_op
    :field temp_compensation: tt64_frame.beacon.temp_compensation
    :field reset_counter: tt64_frame.beacon.reset_counter
    :field uplink_error: tt64_frame.beacon.uplink_error
    :field obc_sent_packet_counter_between_s_beacons: tt64_frame.beacon.obc_sent_packet_counter_between_s_beacons
    :field beacon_interval: tt64_frame.beacon.beacon_interval
    :field sid_s: tt64_frame.beacon.sid_s
    :field tx_sel_reason: tt64_frame.beacon.tx_sel_reason
    :field reason_remote: tt64_frame.beacon.reason_remote
    :field gs_cmd_counter: tt64_frame.beacon.gs_cmd_counter
    :field beacon_count: tt64_frame.beacon.beacon_count
    :field prim_freq_start: tt64_frame.beacon.prim_freq_start
    :field sec_freq_start: tt64_frame.beacon.sec_freq_start
    :field usp: tt64_frame.beacon.usp
    :field rssi_idle: tt64_frame.beacon.rssi_idle
    :field rssi_rx: tt64_frame.beacon.rssi_rx
    :field primary_carrier: tt64_frame.beacon.primary_carrier
    :field secondary_carrier: tt64_frame.beacon.secondary_carrier
    :field used_carrier: tt64_frame.beacon.used_carrier
    :field temperature_compensation_carrier: tt64_frame.beacon.temperature_compensation_carrier
    :field s_time: tt64_frame.beacon.s_time
    :field temp_bat1sw: tt64_frame.beacon.temp_bat1sw
    :field temp_5v: tt64_frame.beacon.temp_5v
    :field eps_version: tt64_frame.beacon.eps_version
    :field sid_e: tt64_frame.beacon.sid_e
    :field temp_bat1: tt64_frame.beacon.temp_bat1
    :field temp_bat2: tt64_frame.beacon.temp_bat2
    :field status_1_3v3_1: tt64_frame.beacon.status_1_3v3_1
    :field status_1_3v3_2: tt64_frame.beacon.status_1_3v3_2
    :field status_1_3v3_3: tt64_frame.beacon.status_1_3v3_3
    :field status_1_3v3_bu: tt64_frame.beacon.status_1_3v3_bu
    :field status_1_5v_1: tt64_frame.beacon.status_1_5v_1
    :field status_1_5v_2: tt64_frame.beacon.status_1_5v_2
    :field status_1_5v_3: tt64_frame.beacon.status_1_5v_3
    :field status_1_5v_4: tt64_frame.beacon.status_1_5v_4
    :field status_2_power_low: tt64_frame.beacon.status_2_power_low
    :field status_2_bat1_pv1: tt64_frame.beacon.status_2_bat1_pv1
    :field status_2_bat2_pv2: tt64_frame.beacon.status_2_bat2_pv2
    :field status_2_3v3: tt64_frame.beacon.status_2_3v3
    :field status_2_5v: tt64_frame.beacon.status_2_5v
    :field status_2_mode: tt64_frame.beacon.status_2_mode
    :field status_3_3v3_burst: tt64_frame.beacon.status_3_3v3_burst
    :field status_3_5v_burst: tt64_frame.beacon.status_3_5v_burst
    :field status_3_bat1_pv2: tt64_frame.beacon.status_3_bat1_pv2
    :field status_3_bat2_pv1: tt64_frame.beacon.status_3_bat2_pv1
    :field status_3_temp_warning: tt64_frame.beacon.status_3_temp_warning
    :field status_3_cc1: tt64_frame.beacon.status_3_cc1
    :field status_3_cc2: tt64_frame.beacon.status_3_cc2
    :field status_3_rbf: tt64_frame.beacon.status_3_rbf
    :field s_beacon_count: tt64_frame.beacon.s_beacon_count
    :field reboot_mc: tt64_frame.beacon.reboot_mc
    :field reboot_cc1: tt64_frame.beacon.reboot_cc1
    :field reboot_cc2: tt64_frame.beacon.reboot_cc2
    :field temp_cc1: tt64_frame.beacon.temp_cc1
    :field temp_cc2: tt64_frame.beacon.temp_cc2
    :field status_cc1_mode: tt64_frame.beacon.status_cc1_mode
    :field status_cc1_mc_timeout: tt64_frame.beacon.status_cc1_mc_timeout
    :field status_cc1_tbd: tt64_frame.beacon.status_cc1_tbd
    :field status_cc1_en_i2c: tt64_frame.beacon.status_cc1_en_i2c
    :field status_cc1_bat1_pv1: tt64_frame.beacon.status_cc1_bat1_pv1
    :field status_cc1_bat2_pv2: tt64_frame.beacon.status_cc1_bat2_pv2
    :field status_cc1_3v3_backup: tt64_frame.beacon.status_cc1_3v3_backup
    :field status_cc2_mode: tt64_frame.beacon.status_cc2_mode
    :field status_cc2_mc_timeout: tt64_frame.beacon.status_cc2_mc_timeout
    :field status_cc2_tbd: tt64_frame.beacon.status_cc2_tbd
    :field status_cc2_en_i2c: tt64_frame.beacon.status_cc2_en_i2c
    :field status_cc2_bat1_pv1: tt64_frame.beacon.status_cc2_bat1_pv1
    :field status_cc2_bat2_pv2: tt64_frame.beacon.status_cc2_bat2_pv2
    :field status_cc2_3v3_backup: tt64_frame.beacon.status_cc2_3v3_backup
    :field v_pv2: tt64_frame.beacon.v_pv2
    :field v_5v_in: tt64_frame.beacon.v_5v_in
    :field v_pv1: tt64_frame.beacon.v_pv1
    :field v_3v3_in: tt64_frame.beacon.v_3v3_in
    :field v_3v3_out: tt64_frame.beacon.v_3v3_out
    :field v_hv: tt64_frame.beacon.v_hv
    :field v_5v_out: tt64_frame.beacon.v_5v_out
    :field v_bat1: tt64_frame.beacon.v_bat1
    :field v_bat2: tt64_frame.beacon.v_bat2
    :field vcc_cc2: tt64_frame.beacon.vcc_cc2
    :field vcc_cc1: tt64_frame.beacon.vcc_cc1
    :field i_pv1_5v: tt64_frame.beacon.i_pv1_5v
    :field i_pv2_5v: tt64_frame.beacon.i_pv2_5v
    :field i_pv1_3v3: tt64_frame.beacon.i_pv1_3v3
    :field i_pv2_3v3: tt64_frame.beacon.i_pv2_3v3
    :field i_pv1_bat1: tt64_frame.beacon.i_pv1_bat1
    :field i_pv2_bat1: tt64_frame.beacon.i_pv2_bat1
    :field i_pv1_bat2: tt64_frame.beacon.i_pv1_bat2
    :field i_pv2_bat2: tt64_frame.beacon.i_pv2_bat2
    :field ant_deploy_status_mb: tt64_frame.beacon.ant_deploy_status_mb
    :field ant_deploy_status_lb: tt64_frame.beacon.ant_deploy_status_lb
    :field temp_comp_hw: tt64_frame.beacon.temp_comp_hw
    :field rbf_state: tt64_frame.beacon.rbf_state
    :field sid_a: tt64_frame.beacon.sid_a
    :field stacie_version: tt64_frame.beacon.stacie_version
    :field ant_temp: tt64_frame.beacon.ant_temp
    :field temp_a: tt64_frame.beacon.temp_a
    :field temp_c: tt64_frame.beacon.temp_c
    :field state_machine_su_script: tt64_frame.beacon.state_machine_su_script
    :field state_machine_su_powered: tt64_frame.beacon.state_machine_su_powered
    :field state_machine_adcs: tt64_frame.beacon.state_machine_adcs
    :field state_machine_not_used: tt64_frame.beacon.state_machine_not_used
    :field state_machine_obc: tt64_frame.beacon.state_machine_obc
    :field mission_counter: tt64_frame.beacon.mission_counter
    :field mission_counter_2: tt64_frame.beacon.mission_counter_2
    :field rssi_a: tt64_frame.beacon.rssi_a
    :field rssi_b: tt64_frame.beacon.rssi_b
    :field stacie_mode_a: tt64_frame.beacon.stacie_mode_a
    :field stacie_mode_c: tt64_frame.beacon.stacie_mode_c
    :field status_task_sensors_running: tt64_frame.beacon.status_task_sensors_running
    :field status_obc_3v3_spa_enabled: tt64_frame.beacon.status_obc_3v3_spa_enabled
    :field status_obc_power_saving_mode: tt64_frame.beacon.status_obc_power_saving_mode
    :field status_eps_cc_used: tt64_frame.beacon.status_eps_cc_used
    :field status_last_reset_source: tt64_frame.beacon.status_last_reset_source
    :field status_power_source: tt64_frame.beacon.status_power_source
    :field status_crystal_osicallation_in_use: tt64_frame.beacon.status_crystal_osicallation_in_use
    :field status_ssp1_initialized: tt64_frame.beacon.status_ssp1_initialized
    :field status_ssp0_initialized: tt64_frame.beacon.status_ssp0_initialized
    :field status_i2c2_initialized: tt64_frame.beacon.status_i2c2_initialized
    :field status_i2c1_initialized: tt64_frame.beacon.status_i2c1_initialized
    :field status_i2c0_initialized: tt64_frame.beacon.status_i2c0_initialized
    :field status_rtc_synchronized: tt64_frame.beacon.status_rtc_synchronized
    :field status_statemachine_initialized: tt64_frame.beacon.status_statemachine_initialized
    :field status_task_maintenance_running: tt64_frame.beacon.status_task_maintenance_running
    :field status_uart_ttc1_initialized: tt64_frame.beacon.status_uart_ttc1_initialized
    :field status_uart_mnlp_initialized: tt64_frame.beacon.status_uart_mnlp_initialized
    :field status_uart_ttc2_initialized: tt64_frame.beacon.status_uart_ttc2_initialized
    :field status_uart_gps_initialized: tt64_frame.beacon.status_uart_gps_initialized
    :field status_adc_initialized: tt64_frame.beacon.status_adc_initialized
    :field status_rtc_initialized: tt64_frame.beacon.status_rtc_initialized
    :field status_i2c_switches_initialized: tt64_frame.beacon.status_i2c_switches_initialized
    :field status_supply_switches_initialized: tt64_frame.beacon.status_supply_switches_initialized
    :field status_eeprom3_initialized: tt64_frame.beacon.status_eeprom3_initialized
    :field status_eeprom2_initialized: tt64_frame.beacon.status_eeprom2_initialized
    :field status_eeprom1_initialized: tt64_frame.beacon.status_eeprom1_initialized
    :field status_eps_cc2_operational: tt64_frame.beacon.status_eps_cc2_operational
    :field status_eps_cc1_opeational: tt64_frame.beacon.status_eps_cc1_opeational
    :field status_timer1_initialized: tt64_frame.beacon.status_timer1_initialized
    :field status_watchdog_initialized: tt64_frame.beacon.status_watchdog_initialized
    :field status_timer0_initialized: tt64_frame.beacon.status_timer0_initialized
    :field status_mpu_initialized: tt64_frame.beacon.status_mpu_initialized
    :field status_onboard_tmp100_initialized: tt64_frame.beacon.status_onboard_tmp100_initialized
    :field status_onboard_mag_initialized: tt64_frame.beacon.status_onboard_mag_initialized
    :field status_msp_initialized: tt64_frame.beacon.status_msp_initialized
    :field status_gyro2_initialized: tt64_frame.beacon.status_gyro2_initialized
    :field status_gyro1_initialized: tt64_frame.beacon.status_gyro1_initialized
    :field status_mag_bp_boom_initialized: tt64_frame.beacon.status_mag_bp_boom_initialized
    :field status_mag_bp_initialized: tt64_frame.beacon.status_mag_bp_initialized
    :field status_bp_initialized: tt64_frame.beacon.status_bp_initialized
    :field status_sa_initailized: tt64_frame.beacon.status_sa_initailized
    :field status_spd_initialized: tt64_frame.beacon.status_spd_initialized
    :field status_spc_initialized: tt64_frame.beacon.status_spc_initialized
    :field status_spb_initialized: tt64_frame.beacon.status_spb_initialized
    :field status_spa_initialized: tt64_frame.beacon.status_spa_initialized
    :field status_flash2_initialized: tt64_frame.beacon.status_flash2_initialized
    :field status_flash1_initialized: tt64_frame.beacon.status_flash1_initialized
    :field status_spd_vcc_on: tt64_frame.beacon.status_spd_vcc_on
    :field status_spc_vcc_on: tt64_frame.beacon.status_spc_vcc_on
    :field status_spb_vcc_on: tt64_frame.beacon.status_spb_vcc_on
    :field status_spa_vcc_on: tt64_frame.beacon.status_spa_vcc_on
    :field status_science_module_initialized: tt64_frame.beacon.status_science_module_initialized
    :field status_ttc2_initialized: tt64_frame.beacon.status_ttc2_initialized
    :field status_ttc1_initialized: tt64_frame.beacon.status_ttc1_initialized
    :field status_gps_initialized: tt64_frame.beacon.status_gps_initialized
    :field status_onboard_mag_powersafe: tt64_frame.beacon.status_onboard_mag_powersafe
    :field status_i2c_sw_d_on: tt64_frame.beacon.status_i2c_sw_d_on
    :field status_i2c_sw_c_on: tt64_frame.beacon.status_i2c_sw_c_on
    :field status_i2c_sw_b_on: tt64_frame.beacon.status_i2c_sw_b_on
    :field status_i2c_sw_a_on: tt64_frame.beacon.status_i2c_sw_a_on
    :field status_sa_vcc_on: tt64_frame.beacon.status_sa_vcc_on
    :field status_bp2_vcc_on: tt64_frame.beacon.status_bp2_vcc_on
    :field status_bp1_vcc_on: tt64_frame.beacon.status_bp1_vcc_on
    :field status_eeprom_page_cycle_overflow: tt64_frame.beacon.status_eeprom_page_cycle_overflow
    :field status_rtc_oscillator_error: tt64_frame.beacon.status_rtc_oscillator_error
    :field status_mnlp_5v_enabled: tt64_frame.beacon.status_mnlp_5v_enabled
    :field status_mag_bp_boom_power_saving_mode: tt64_frame.beacon.status_mag_bp_boom_power_saving_mode
    :field status_mag_bp_power_saving_mode: tt64_frame.beacon.status_mag_bp_power_saving_mode
    :field status_tmp100_powersafe: tt64_frame.beacon.status_tmp100_powersafe
    :field status_mpu_powersafe: tt64_frame.beacon.status_mpu_powersafe
    :field status_gyro_powersafe: tt64_frame.beacon.status_gyro_powersafe
    :field status_default_config_used: tt64_frame.beacon.status_default_config_used
    :field status_timer1_running: tt64_frame.beacon.status_timer1_running
    :field status_timer0_running: tt64_frame.beacon.status_timer0_running
    :field status_i2c2_frequent_errors: tt64_frame.beacon.status_i2c2_frequent_errors
    :field status_i2c1_frequent_errors: tt64_frame.beacon.status_i2c1_frequent_errors
    :field status_i2c0_frequent_errors: tt64_frame.beacon.status_i2c0_frequent_errors
    :field status_ssp1_frequent_errors: tt64_frame.beacon.status_ssp1_frequent_errors
    :field status_ssp0_frequent_errors: tt64_frame.beacon.status_ssp0_frequent_errors
    :field error_code: tt64_frame.beacon.error_code
    :field error_code_before_reset: tt64_frame.beacon.error_code_before_reset
    :field resets_counter: tt64_frame.beacon.resets_counter
    :field script_slot_cs1: tt64_frame.beacon.script_slot_cs1
    :field script_slot_su7: tt64_frame.beacon.script_slot_su7
    :field script_slot_su6: tt64_frame.beacon.script_slot_su6
    :field script_slot_su5: tt64_frame.beacon.script_slot_su5
    :field script_slot_su4: tt64_frame.beacon.script_slot_su4
    :field script_slot_su3: tt64_frame.beacon.script_slot_su3
    :field script_slot_su2: tt64_frame.beacon.script_slot_su2
    :field script_slot_su1: tt64_frame.beacon.script_slot_su1
    :field script_slot_cs5: tt64_frame.beacon.script_slot_cs5
    :field script_slot_cs4: tt64_frame.beacon.script_slot_cs4
    :field script_slot_cs3: tt64_frame.beacon.script_slot_cs3
    :field script_slot_cs2: tt64_frame.beacon.script_slot_cs2
    :field temp_sp_x_minus: tt64_frame.beacon.temp_sp_x_minus
    :field temp_sp_x_plus: tt64_frame.beacon.temp_sp_x_plus
    :field temp_sp_y_minus: tt64_frame.beacon.temp_sp_y_minus
    :field temp_sp_y_plus: tt64_frame.beacon.temp_sp_y_plus
    
    .. seealso::
       Source - https://spacedatacenter.at/pegasus/img/hamoperatormanual12.pdf
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.tt64_frame = Pegasus.Tt64Frame(self._io, self, self._root)

    class HkdDownload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.padding_hkd1 = self._io.read_bytes(10)
            self.v_pv1_raw = self._io.read_u1()
            self.v_pv2_raw = self._io.read_u1()
            self.v_5v_in_raw = self._io.read_u1()
            self.v_3v3_in_raw = self._io.read_u1()
            self.v_5v_out_raw = self._io.read_u1()
            self.v_3v3_out_raw = self._io.read_u1()
            self.i_pv1_5v_raw = self._io.read_u1()
            self.i_pv2_5v_raw = self._io.read_u1()
            self.i_pv1_3v3_raw = self._io.read_u1()
            self.i_pv2_3v3_raw = self._io.read_u1()
            self.temp_bat1sw = self._io.read_s1()
            self.temp_5v = self._io.read_s1()
            self.v_hv_raw = self._io.read_u1()
            self.i_pv1_bat1_raw = self._io.read_u1()
            self.i_pv2_bat1_raw = self._io.read_u1()
            self.i_pv1_bat2_raw = self._io.read_u1()
            self.i_pv2_bat2_raw = self._io.read_u1()
            self.v_bat1_raw = self._io.read_u1()
            self.v_bat2_raw = self._io.read_u1()
            self.vcc_cc2_raw = self._io.read_u1()
            self.vcc_cc1_raw = self._io.read_u1()
            self.temp_bat1 = self._io.read_s1()
            self.temp_bat2 = self._io.read_s1()
            self.status_1_3v3_1 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_2 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_3 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_bu = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_1 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_2 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_3 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_4 = self._io.read_bits_int_be(1) != 0
            self.status_2_power_low = self._io.read_bits_int_be(1) != 0
            self.status_2_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_2_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_2_3v3 = self._io.read_bits_int_be(1) != 0
            self.status_2_5v = self._io.read_bits_int_be(1) != 0
            self.status_2_mode = self._io.read_bits_int_be(3)
            self.status_3_3v3_burst = self._io.read_bits_int_be(1) != 0
            self.status_3_5v_burst = self._io.read_bits_int_be(1) != 0
            self.status_3_bat1_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_3_bat2_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_3_temp_warning = self._io.read_bits_int_be(1) != 0
            self.status_3_cc1 = self._io.read_bits_int_be(1) != 0
            self.status_3_cc2 = self._io.read_bits_int_be(1) != 0
            self.status_3_rbf = self._io.read_bits_int_be(1) != 0
            self.status_cc1_mode = self._io.read_bits_int_be(2)
            self.status_cc1_mc_timeout = self._io.read_bits_int_be(1) != 0
            self.status_cc1_tbd = self._io.read_bits_int_be(1) != 0
            self.status_cc1_en_i2c = self._io.read_bits_int_be(1) != 0
            self.status_cc1_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_cc1_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_cc1_3v3_backup = self._io.read_bits_int_be(1) != 0
            self.status_cc2_mode = self._io.read_bits_int_be(2)
            self.status_cc2_mc_timeout = self._io.read_bits_int_be(1) != 0
            self.status_cc2_tbd = self._io.read_bits_int_be(1) != 0
            self.status_cc2_en_i2c = self._io.read_bits_int_be(1) != 0
            self.status_cc2_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_cc2_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_cc2_3v3_backup = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.temp_a = self._io.read_s1()
            self.temp_c = self._io.read_s1()
            self.rssi_a_raw = self._io.read_u1()
            self.rssi_b_raw = self._io.read_u1()
            self.state_machine_su_script = self._io.read_bits_int_be(1) != 0
            self.state_machine_su_powered = self._io.read_bits_int_be(1) != 0
            self.state_machine_adcs = self._io.read_bits_int_be(1) != 0
            self.state_machine_not_used = self._io.read_bits_int_be(1) != 0
            self.state_machine_obc = self._io.read_bits_int_be(4)
            self._io.align_to_byte()
            self.mission_counter = self._io.read_u1()
            self.mission_counter_2 = self._io.read_u1()

        @property
        def vcc_cc2(self):
            """Supply voltage of CC2 (communication controller 2) [V]."""
            if hasattr(self, '_m_vcc_cc2'):
                return self._m_vcc_cc2

            self._m_vcc_cc2 = (((((((((self.vcc_cc2_raw & 128) // 128 * 4) + ((self.vcc_cc2_raw & 64) // 64 * 2)) + ((self.vcc_cc2_raw & 32) // 32 * 1)) + ((self.vcc_cc2_raw & 16) // 16 * 0.5)) + ((self.vcc_cc2_raw & 8) // 8 * 0.25)) + ((self.vcc_cc2_raw & 4) // 4 * 0.125)) + ((self.vcc_cc2_raw & 2) // 2 * 0.0625)) + ((self.vcc_cc2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_vcc_cc2', None)

        @property
        def v_bat2(self):
            """Voltage of the battery 2 [V]."""
            if hasattr(self, '_m_v_bat2'):
                return self._m_v_bat2

            self._m_v_bat2 = (((((((((self.v_bat2_raw & 128) // 128 * 4) + ((self.v_bat2_raw & 64) // 64 * 2)) + ((self.v_bat2_raw & 32) // 32 * 1)) + ((self.v_bat2_raw & 16) // 16 * 0.5)) + ((self.v_bat2_raw & 8) // 8 * 0.25)) + ((self.v_bat2_raw & 4) // 4 * 0.125)) + ((self.v_bat2_raw & 2) // 2 * 0.0625)) + ((self.v_bat2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_bat2', None)

        @property
        def rssi_a(self):
            """Received Signal Strength Indication of TRX module A during the last uplink [dBm]."""
            if hasattr(self, '_m_rssi_a'):
                return self._m_rssi_a

            self._m_rssi_a = (-132.0 + self.rssi_a_raw // 2)
            return getattr(self, '_m_rssi_a', None)

        @property
        def rssi_b(self):
            """Received Signal Strength Indication of TRX module C during the last uplink [dBm]."""
            if hasattr(self, '_m_rssi_b'):
                return self._m_rssi_b

            self._m_rssi_b = (-132.0 + self.rssi_b_raw // 2)
            return getattr(self, '_m_rssi_b', None)

        @property
        def v_pv1(self):
            """Voltage at PV1-bus (Solarbus 1) [V]."""
            if hasattr(self, '_m_v_pv1'):
                return self._m_v_pv1

            self._m_v_pv1 = (((((((((self.v_pv1_raw & 128) // 128 * 4) + ((self.v_pv1_raw & 64) // 64 * 2)) + ((self.v_pv1_raw & 32) // 32 * 1)) + ((self.v_pv1_raw & 16) // 16 * 0.5)) + ((self.v_pv1_raw & 8) // 8 * 0.25)) + ((self.v_pv1_raw & 4) // 4 * 0.125)) + ((self.v_pv1_raw & 2) // 2 * 0.0625)) + ((self.v_pv1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_pv1', None)

        @property
        def v_hv(self):
            """Voltage at the output of the HV supply to the PPTs measured at FET4-2 [V]."""
            if hasattr(self, '_m_v_hv'):
                return self._m_v_hv

            self._m_v_hv = (((((((((self.v_hv_raw & 128) // 128 * 4) + ((self.v_hv_raw & 64) // 64 * 2)) + ((self.v_hv_raw & 32) // 32 * 1)) + ((self.v_hv_raw & 16) // 16 * 0.5)) + ((self.v_hv_raw & 8) // 8 * 0.25)) + ((self.v_hv_raw & 4) // 4 * 0.125)) + ((self.v_hv_raw & 2) // 2 * 0.0625)) + ((self.v_hv_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_hv', None)

        @property
        def v_bat1(self):
            """Voltage of the battery 1 [V]."""
            if hasattr(self, '_m_v_bat1'):
                return self._m_v_bat1

            self._m_v_bat1 = (((((((((self.v_bat1_raw & 128) // 128 * 4) + ((self.v_bat1_raw & 64) // 64 * 2)) + ((self.v_bat1_raw & 32) // 32 * 1)) + ((self.v_bat1_raw & 16) // 16 * 0.5)) + ((self.v_bat1_raw & 8) // 8 * 0.25)) + ((self.v_bat1_raw & 4) // 4 * 0.125)) + ((self.v_bat1_raw & 2) // 2 * 0.0625)) + ((self.v_bat1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_bat1', None)

        @property
        def i_pv1_bat2(self):
            """Current through FET2-1 between PV1-bus and battery 2 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv1_bat2'):
                return self._m_i_pv1_bat2

            self._m_i_pv1_bat2 = (((((((((self.i_pv1_bat2_raw & 128) // 128 * -8) + ((self.i_pv1_bat2_raw & 64) // 64 * 4)) + ((self.i_pv1_bat2_raw & 32) // 32 * 2)) + ((self.i_pv1_bat2_raw & 16) // 16 * 1)) + ((self.i_pv1_bat2_raw & 8) // 8 * 0.5)) + ((self.i_pv1_bat2_raw & 4) // 4 * 0.25)) + ((self.i_pv1_bat2_raw & 2) // 2 * 0.125)) + ((self.i_pv1_bat2_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_bat2', None)

        @property
        def v_3v3_out(self):
            """Voltage at the output of the 3V3 converter [V]."""
            if hasattr(self, '_m_v_3v3_out'):
                return self._m_v_3v3_out

            self._m_v_3v3_out = (((((((((self.v_3v3_out_raw & 128) // 128 * 4) + ((self.v_3v3_out_raw & 64) // 64 * 2)) + ((self.v_3v3_out_raw & 32) // 32 * 1)) + ((self.v_3v3_out_raw & 16) // 16 * 0.5)) + ((self.v_3v3_out_raw & 8) // 8 * 0.25)) + ((self.v_3v3_out_raw & 4) // 4 * 0.125)) + ((self.v_3v3_out_raw & 2) // 2 * 0.0625)) + ((self.v_3v3_out_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_3v3_out', None)

        @property
        def v_3v3_in(self):
            """Voltage at the input of the 3V3 converter measured at FET5-2 [V]."""
            if hasattr(self, '_m_v_3v3_in'):
                return self._m_v_3v3_in

            self._m_v_3v3_in = (((((((((self.v_3v3_in_raw & 128) // 128 * 4) + ((self.v_3v3_in_raw & 64) // 64 * 2)) + ((self.v_3v3_in_raw & 32) // 32 * 1)) + ((self.v_3v3_in_raw & 16) // 16 * 0.5)) + ((self.v_3v3_in_raw & 8) // 8 * 0.25)) + ((self.v_3v3_in_raw & 4) // 4 * 0.125)) + ((self.v_3v3_in_raw & 2) // 2 * 0.0625)) + ((self.v_3v3_in_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_3v3_in', None)

        @property
        def i_pv1_bat1(self):
            """Current through FET1-1 between PV1-bus and battery 1 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv1_bat1'):
                return self._m_i_pv1_bat1

            self._m_i_pv1_bat1 = (((((((((self.i_pv1_bat1_raw & 128) // 128 * -8) + ((self.i_pv1_bat1_raw & 64) // 64 * 4)) + ((self.i_pv1_bat1_raw & 32) // 32 * 2)) + ((self.i_pv1_bat1_raw & 16) // 16 * 1)) + ((self.i_pv1_bat1_raw & 8) // 8 * 0.5)) + ((self.i_pv1_bat1_raw & 4) // 4 * 0.25)) + ((self.i_pv1_bat1_raw & 2) // 2 * 0.125)) + ((self.i_pv1_bat1_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_bat1', None)

        @property
        def i_pv2_3v3(self):
            """Current through FET5-2 between PV2-bus and 3V3 converter [A]."""
            if hasattr(self, '_m_i_pv2_3v3'):
                return self._m_i_pv2_3v3

            self._m_i_pv2_3v3 = (((((((((self.i_pv2_3v3_raw & 128) // 128 * -8) + ((self.i_pv2_3v3_raw & 64) // 64 * 4)) + ((self.i_pv2_3v3_raw & 32) // 32 * 2)) + ((self.i_pv2_3v3_raw & 16) // 16 * 1)) + ((self.i_pv2_3v3_raw & 8) // 8 * 0.5)) + ((self.i_pv2_3v3_raw & 4) // 4 * 0.25)) + ((self.i_pv2_3v3_raw & 2) // 2 * 0.125)) + ((self.i_pv2_3v3_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_3v3', None)

        @property
        def vcc_cc1(self):
            """Supply voltage of CC1 (communication controller 1) [V]."""
            if hasattr(self, '_m_vcc_cc1'):
                return self._m_vcc_cc1

            self._m_vcc_cc1 = (((((((((self.vcc_cc1_raw & 128) // 128 * 4) + ((self.vcc_cc1_raw & 64) // 64 * 2)) + ((self.vcc_cc1_raw & 32) // 32 * 1)) + ((self.vcc_cc1_raw & 16) // 16 * 0.5)) + ((self.vcc_cc1_raw & 8) // 8 * 0.25)) + ((self.vcc_cc1_raw & 4) // 4 * 0.125)) + ((self.vcc_cc1_raw & 2) // 2 * 0.0625)) + ((self.vcc_cc1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_vcc_cc1', None)

        @property
        def v_5v_out(self):
            """Voltage at the output of the 5V converter [V]."""
            if hasattr(self, '_m_v_5v_out'):
                return self._m_v_5v_out

            self._m_v_5v_out = (((((((((self.v_5v_out_raw & 128) // 128 * 4) + ((self.v_5v_out_raw & 64) // 64 * 2)) + ((self.v_5v_out_raw & 32) // 32 * 1)) + ((self.v_5v_out_raw & 16) // 16 * 0.5)) + ((self.v_5v_out_raw & 8) // 8 * 0.25)) + ((self.v_5v_out_raw & 4) // 4 * 0.125)) + ((self.v_5v_out_raw & 2) // 2 * 0.0625)) + ((self.v_5v_out_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_5v_out', None)

        @property
        def v_5v_in(self):
            """Voltage at the input of the 5V converter measured at FET3-1 [V]."""
            if hasattr(self, '_m_v_5v_in'):
                return self._m_v_5v_in

            self._m_v_5v_in = (((((((((self.v_5v_in_raw & 128) // 128 * 4) + ((self.v_5v_in_raw & 64) // 64 * 2)) + ((self.v_5v_in_raw & 32) // 32 * 1)) + ((self.v_5v_in_raw & 16) // 16 * 0.5)) + ((self.v_5v_in_raw & 8) // 8 * 0.25)) + ((self.v_5v_in_raw & 4) // 4 * 0.125)) + ((self.v_5v_in_raw & 2) // 2 * 0.0625)) + ((self.v_5v_in_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_5v_in', None)

        @property
        def i_pv1_5v(self):
            """Current through FET3-1 between PV1-bus and 5V converter [A]."""
            if hasattr(self, '_m_i_pv1_5v'):
                return self._m_i_pv1_5v

            self._m_i_pv1_5v = (((((((((self.i_pv1_5v_raw & 128) // 128 * -8) + ((self.i_pv1_5v_raw & 64) // 64 * 4)) + ((self.i_pv1_5v_raw & 32) // 32 * 2)) + ((self.i_pv1_5v_raw & 16) // 16 * 1)) + ((self.i_pv1_5v_raw & 8) // 8 * 0.5)) + ((self.i_pv1_5v_raw & 4) // 4 * 0.25)) + ((self.i_pv1_5v_raw & 2) // 2 * 0.125)) + ((self.i_pv1_5v_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_5v', None)

        @property
        def i_pv2_bat2(self):
            """Current through FET2-2 between PV2-bus and battery 2 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv2_bat2'):
                return self._m_i_pv2_bat2

            self._m_i_pv2_bat2 = (((((((((self.i_pv2_bat2_raw & 128) // 128 * -8) + ((self.i_pv2_bat2_raw & 64) // 64 * 4)) + ((self.i_pv2_bat2_raw & 32) // 32 * 2)) + ((self.i_pv2_bat2_raw & 16) // 16 * 1)) + ((self.i_pv2_bat2_raw & 8) // 8 * 0.5)) + ((self.i_pv2_bat2_raw & 4) // 4 * 0.25)) + ((self.i_pv2_bat2_raw & 2) // 2 * 0.125)) + ((self.i_pv2_bat2_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_bat2', None)

        @property
        def i_pv2_bat1(self):
            """Current through FET1-2 between PV2-bus and battery 1 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv2_bat1'):
                return self._m_i_pv2_bat1

            self._m_i_pv2_bat1 = (((((((((self.i_pv2_bat1_raw & 128) // 128 * -8) + ((self.i_pv2_bat1_raw & 64) // 64 * 4)) + ((self.i_pv2_bat1_raw & 32) // 32 * 2)) + ((self.i_pv2_bat1_raw & 16) // 16 * 1)) + ((self.i_pv2_bat1_raw & 8) // 8 * 0.5)) + ((self.i_pv2_bat1_raw & 4) // 4 * 0.25)) + ((self.i_pv2_bat1_raw & 2) // 2 * 0.125)) + ((self.i_pv2_bat1_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_bat1', None)

        @property
        def i_pv1_3v3(self):
            """Current through FET5-1 between PV1-bus and 3V3 converter [A]."""
            if hasattr(self, '_m_i_pv1_3v3'):
                return self._m_i_pv1_3v3

            self._m_i_pv1_3v3 = (((((((((self.i_pv1_3v3_raw & 128) // 128 * -8) + ((self.i_pv1_3v3_raw & 64) // 64 * 4)) + ((self.i_pv1_3v3_raw & 32) // 32 * 2)) + ((self.i_pv1_3v3_raw & 16) // 16 * 1)) + ((self.i_pv1_3v3_raw & 8) // 8 * 0.5)) + ((self.i_pv1_3v3_raw & 4) // 4 * 0.25)) + ((self.i_pv1_3v3_raw & 2) // 2 * 0.125)) + ((self.i_pv1_3v3_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_3v3', None)

        @property
        def v_pv2(self):
            """Voltage at PV2-bus (Solarbus 1) [V]."""
            if hasattr(self, '_m_v_pv2'):
                return self._m_v_pv2

            self._m_v_pv2 = (((((((((self.v_pv2_raw & 128) // 128 * 4) + ((self.v_pv2_raw & 64) // 64 * 2)) + ((self.v_pv2_raw & 32) // 32 * 1)) + ((self.v_pv2_raw & 16) // 16 * 0.5)) + ((self.v_pv2_raw & 8) // 8 * 0.25)) + ((self.v_pv2_raw & 4) // 4 * 0.125)) + ((self.v_pv2_raw & 2) // 2 * 0.0625)) + ((self.v_pv2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_pv2', None)

        @property
        def i_pv2_5v(self):
            """Current through FET3-2 between PV2-bus and 5V converter [A]."""
            if hasattr(self, '_m_i_pv2_5v'):
                return self._m_i_pv2_5v

            self._m_i_pv2_5v = (((((((((self.i_pv2_5v_raw & 128) // 128 * -8) + ((self.i_pv2_5v_raw & 64) // 64 * 4)) + ((self.i_pv2_5v_raw & 32) // 32 * 2)) + ((self.i_pv2_5v_raw & 16) // 16 * 1)) + ((self.i_pv2_5v_raw & 8) // 8 * 0.5)) + ((self.i_pv2_5v_raw & 4) // 4 * 0.25)) + ((self.i_pv2_5v_raw & 2) // 2 * 0.125)) + ((self.i_pv2_5v_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_5v', None)


    class O2Beacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.call = (self._io.read_bytes(6)).decode(u"ASCII")
            self.gps_time_padding = self._io.read_bytes(4)
            self.geo_location_padding = self._io.read_bytes(11)
            self.adcs_status_padding = self._io.read_bytes(1)
            self.adcs_angle_deviation_padding = self._io.read_bytes(1)
            self.status_task_sensors_running = self._io.read_bits_int_be(1) != 0
            self.status_obc_3v3_spa_enabled = self._io.read_bits_int_be(1) != 0
            self.status_obc_power_saving_mode = self._io.read_bits_int_be(1) != 0
            self.status_eps_cc_used = self._io.read_bits_int_be(1) != 0
            self.status_last_reset_source = self._io.read_bits_int_be(2)
            self.status_power_source = self._io.read_bits_int_be(1) != 0
            self.status_crystal_osicallation_in_use = self._io.read_bits_int_be(1) != 0
            self.status_ssp1_initialized = self._io.read_bits_int_be(1) != 0
            self.status_ssp0_initialized = self._io.read_bits_int_be(1) != 0
            self.status_i2c2_initialized = self._io.read_bits_int_be(1) != 0
            self.status_i2c1_initialized = self._io.read_bits_int_be(1) != 0
            self.status_i2c0_initialized = self._io.read_bits_int_be(1) != 0
            self.status_rtc_synchronized = self._io.read_bits_int_be(1) != 0
            self.status_statemachine_initialized = self._io.read_bits_int_be(1) != 0
            self.status_task_maintenance_running = self._io.read_bits_int_be(1) != 0
            self.status_uart_ttc1_initialized = self._io.read_bits_int_be(1) != 0
            self.status_uart_mnlp_initialized = self._io.read_bits_int_be(1) != 0
            self.status_uart_ttc2_initialized = self._io.read_bits_int_be(1) != 0
            self.status_uart_gps_initialized = self._io.read_bits_int_be(1) != 0
            self.status_adc_initialized = self._io.read_bits_int_be(1) != 0
            self.status_rtc_initialized = self._io.read_bits_int_be(1) != 0
            self.status_i2c_switches_initialized = self._io.read_bits_int_be(1) != 0
            self.status_supply_switches_initialized = self._io.read_bits_int_be(1) != 0
            self.status_eeprom3_initialized = self._io.read_bits_int_be(1) != 0
            self.status_eeprom2_initialized = self._io.read_bits_int_be(1) != 0
            self.status_eeprom1_initialized = self._io.read_bits_int_be(1) != 0
            self.status_eps_cc2_operational = self._io.read_bits_int_be(1) != 0
            self.status_eps_cc1_opeational = self._io.read_bits_int_be(1) != 0
            self.status_timer1_initialized = self._io.read_bits_int_be(1) != 0
            self.status_watchdog_initialized = self._io.read_bits_int_be(1) != 0
            self.status_timer0_initialized = self._io.read_bits_int_be(1) != 0
            self.status_mpu_initialized = self._io.read_bits_int_be(1) != 0
            self.status_onboard_tmp100_initialized = self._io.read_bits_int_be(1) != 0
            self.status_onboard_mag_initialized = self._io.read_bits_int_be(1) != 0
            self.status_msp_initialized = self._io.read_bits_int_be(1) != 0
            self.status_gyro2_initialized = self._io.read_bits_int_be(1) != 0
            self.status_gyro1_initialized = self._io.read_bits_int_be(1) != 0
            self.status_mag_bp_boom_initialized = self._io.read_bits_int_be(1) != 0
            self.status_mag_bp_initialized = self._io.read_bits_int_be(1) != 0
            self.status_bp_initialized = self._io.read_bits_int_be(1) != 0
            self.status_sa_initailized = self._io.read_bits_int_be(1) != 0
            self.status_spd_initialized = self._io.read_bits_int_be(1) != 0
            self.status_spc_initialized = self._io.read_bits_int_be(1) != 0
            self.status_spb_initialized = self._io.read_bits_int_be(1) != 0
            self.status_spa_initialized = self._io.read_bits_int_be(1) != 0
            self.status_flash2_initialized = self._io.read_bits_int_be(1) != 0
            self.status_flash1_initialized = self._io.read_bits_int_be(1) != 0
            self.status_spd_vcc_on = self._io.read_bits_int_be(1) != 0
            self.status_spc_vcc_on = self._io.read_bits_int_be(1) != 0
            self.status_spb_vcc_on = self._io.read_bits_int_be(1) != 0
            self.status_spa_vcc_on = self._io.read_bits_int_be(1) != 0
            self.status_science_module_initialized = self._io.read_bits_int_be(1) != 0
            self.status_ttc2_initialized = self._io.read_bits_int_be(1) != 0
            self.status_ttc1_initialized = self._io.read_bits_int_be(1) != 0
            self.status_gps_initialized = self._io.read_bits_int_be(1) != 0
            self.status_onboard_mag_powersafe = self._io.read_bits_int_be(1) != 0
            self.status_i2c_sw_d_on = self._io.read_bits_int_be(1) != 0
            self.status_i2c_sw_c_on = self._io.read_bits_int_be(1) != 0
            self.status_i2c_sw_b_on = self._io.read_bits_int_be(1) != 0
            self.status_i2c_sw_a_on = self._io.read_bits_int_be(1) != 0
            self.status_sa_vcc_on = self._io.read_bits_int_be(1) != 0
            self.status_bp2_vcc_on = self._io.read_bits_int_be(1) != 0
            self.status_bp1_vcc_on = self._io.read_bits_int_be(1) != 0
            self.status_eeprom_page_cycle_overflow = self._io.read_bits_int_be(1) != 0
            self.status_rtc_oscillator_error = self._io.read_bits_int_be(1) != 0
            self.status_mnlp_5v_enabled = self._io.read_bits_int_be(1) != 0
            self.status_mag_bp_boom_power_saving_mode = self._io.read_bits_int_be(1) != 0
            self.status_mag_bp_power_saving_mode = self._io.read_bits_int_be(1) != 0
            self.status_tmp100_powersafe = self._io.read_bits_int_be(1) != 0
            self.status_mpu_powersafe = self._io.read_bits_int_be(1) != 0
            self.status_gyro_powersafe = self._io.read_bits_int_be(1) != 0
            self.status_default_config_used = self._io.read_bits_int_be(1) != 0
            self.status_timer1_running = self._io.read_bits_int_be(1) != 0
            self.status_timer0_running = self._io.read_bits_int_be(1) != 0
            self.status_i2c2_frequent_errors = self._io.read_bits_int_be(1) != 0
            self.status_i2c1_frequent_errors = self._io.read_bits_int_be(1) != 0
            self.status_i2c0_frequent_errors = self._io.read_bits_int_be(1) != 0
            self.status_ssp1_frequent_errors = self._io.read_bits_int_be(1) != 0
            self.status_ssp0_frequent_errors = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.error_code = self._io.read_u1()
            self.error_code_before_reset = self._io.read_u1()
            self.resets_counter = self._io.read_u4le()
            self.temp_sp_x_minus_raw = self._io.read_u1()
            self.temp_sp_x_plus_raw = self._io.read_u1()
            self.temp_sp_y_minus_raw = self._io.read_u1()
            self.temp_sp_y_plus_raw = self._io.read_u1()
            self.script_slot_cs1 = self._io.read_bits_int_be(1) != 0
            self.script_slot_su7 = self._io.read_bits_int_be(1) != 0
            self.script_slot_su6 = self._io.read_bits_int_be(1) != 0
            self.script_slot_su5 = self._io.read_bits_int_be(1) != 0
            self.script_slot_su4 = self._io.read_bits_int_be(1) != 0
            self.script_slot_su3 = self._io.read_bits_int_be(1) != 0
            self.script_slot_su2 = self._io.read_bits_int_be(1) != 0
            self.script_slot_su1 = self._io.read_bits_int_be(1) != 0
            self.script_slot_padding = self._io.read_bits_int_be(4)
            self.script_slot_cs5 = self._io.read_bits_int_be(1) != 0
            self.script_slot_cs4 = self._io.read_bits_int_be(1) != 0
            self.script_slot_cs3 = self._io.read_bits_int_be(1) != 0
            self.script_slot_cs2 = self._io.read_bits_int_be(1) != 0

        @property
        def temp_sp_x_minus(self):
            """Temperature sidepanel X- [°C]."""
            if hasattr(self, '_m_temp_sp_x_minus'):
                return self._m_temp_sp_x_minus

            self._m_temp_sp_x_minus = ((self.temp_sp_x_minus_raw * 2.65) - 273.15)
            return getattr(self, '_m_temp_sp_x_minus', None)

        @property
        def temp_sp_x_plus(self):
            """Temperature sidepanel X+ [°C]."""
            if hasattr(self, '_m_temp_sp_x_plus'):
                return self._m_temp_sp_x_plus

            self._m_temp_sp_x_plus = ((self.temp_sp_x_plus_raw * 2.65) - 273.15)
            return getattr(self, '_m_temp_sp_x_plus', None)

        @property
        def temp_sp_y_minus(self):
            """Temperature sidepanel Y- [°C]."""
            if hasattr(self, '_m_temp_sp_y_minus'):
                return self._m_temp_sp_y_minus

            self._m_temp_sp_y_minus = ((self.temp_sp_y_minus_raw * 2.65) - 273.15)
            return getattr(self, '_m_temp_sp_y_minus', None)

        @property
        def temp_sp_y_plus(self):
            """Temperature sidepanel Y+ [°C]."""
            if hasattr(self, '_m_temp_sp_y_plus'):
                return self._m_temp_sp_y_plus

            self._m_temp_sp_y_plus = ((self.temp_sp_y_plus_raw * 2.65) - 273.15)
            return getattr(self, '_m_temp_sp_y_plus', None)


    class Tt64Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self.pid
            if _on == 192:
                self.beacon = Pegasus.SBeacon(self._io, self, self._root)
            elif _on == 82:
                self.beacon = Pegasus.HkdDownload(self._io, self, self._root)
            elif _on == 86:
                self.beacon = Pegasus.O2Beacon(self._io, self, self._root)
            elif _on == 83:
                self.beacon = Pegasus.O1Beacon(self._io, self, self._root)
            elif _on == 195:
                self.beacon = Pegasus.ABeacon(self._io, self, self._root)
            elif _on == 193:
                self.beacon = Pegasus.EBeacon(self._io, self, self._root)


    class O1Beacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.call = (self._io.read_bytes(6)).decode(u"ASCII")
            self.v_pv1_raw = self._io.read_u1()
            self.v_pv2_raw = self._io.read_u1()
            self.v_5v_in_raw = self._io.read_u1()
            self.v_3v3_in_raw = self._io.read_u1()
            self.v_5v_out_raw = self._io.read_u1()
            self.v_3v3_out_raw = self._io.read_u1()
            self.i_pv1_5v_raw = self._io.read_u1()
            self.i_pv2_5v_raw = self._io.read_u1()
            self.i_pv1_3v3_raw = self._io.read_u1()
            self.i_pv2_3v3_raw = self._io.read_u1()
            self.temp_bat1sw = self._io.read_s1()
            self.temp_5v = self._io.read_s1()
            self.v_hv_raw = self._io.read_u1()
            self.i_pv1_bat1_raw = self._io.read_u1()
            self.i_pv2_bat1_raw = self._io.read_u1()
            self.i_pv1_bat2_raw = self._io.read_u1()
            self.i_pv2_bat2_raw = self._io.read_u1()
            self.v_bat1_raw = self._io.read_u1()
            self.v_bat2_raw = self._io.read_u1()
            self.vcc_cc2_raw = self._io.read_u1()
            self.vcc_cc1_raw = self._io.read_u1()
            self.temp_bat1 = self._io.read_s1()
            self.temp_bat2 = self._io.read_s1()
            self.status_1_3v3_1 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_2 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_3 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_bu = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_1 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_2 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_3 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_4 = self._io.read_bits_int_be(1) != 0
            self.status_2_power_low = self._io.read_bits_int_be(1) != 0
            self.status_2_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_2_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_2_3v3 = self._io.read_bits_int_be(1) != 0
            self.status_2_5v = self._io.read_bits_int_be(1) != 0
            self.status_2_mode = self._io.read_bits_int_be(3)
            self.status_3_3v3_burst = self._io.read_bits_int_be(1) != 0
            self.status_3_5v_burst = self._io.read_bits_int_be(1) != 0
            self.status_3_bat1_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_3_bat2_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_3_temp_warning = self._io.read_bits_int_be(1) != 0
            self.status_3_cc1 = self._io.read_bits_int_be(1) != 0
            self.status_3_cc2 = self._io.read_bits_int_be(1) != 0
            self.status_3_rbf = self._io.read_bits_int_be(1) != 0
            self.status_cc1_mode = self._io.read_bits_int_be(2)
            self.status_cc1_mc_timeout = self._io.read_bits_int_be(1) != 0
            self.status_cc1_tbd = self._io.read_bits_int_be(1) != 0
            self.status_cc1_en_i2c = self._io.read_bits_int_be(1) != 0
            self.status_cc1_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_cc1_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_cc1_3v3_backup = self._io.read_bits_int_be(1) != 0
            self.status_cc2_mode = self._io.read_bits_int_be(2)
            self.status_cc2_mc_timeout = self._io.read_bits_int_be(1) != 0
            self.status_cc2_tbd = self._io.read_bits_int_be(1) != 0
            self.status_cc2_en_i2c = self._io.read_bits_int_be(1) != 0
            self.status_cc2_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_cc2_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_cc2_3v3_backup = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.reboot_mc = self._io.read_u1()
            self.reboot_cc1 = self._io.read_u1()
            self.reboot_cc2 = self._io.read_u1()
            self.temp_a = self._io.read_s1()
            self.temp_c = self._io.read_s1()
            self.rssi_a_raw = self._io.read_u1()
            self.rssi_b_raw = self._io.read_u1()
            self.stacie_mode_a = self._io.read_bits_int_be(4)
            self.stacie_mode_c = self._io.read_bits_int_be(4)
            self.state_machine_su_script = self._io.read_bits_int_be(1) != 0
            self.state_machine_su_powered = self._io.read_bits_int_be(1) != 0
            self.state_machine_adcs = self._io.read_bits_int_be(1) != 0
            self.state_machine_not_used = self._io.read_bits_int_be(1) != 0
            self.state_machine_obc = self._io.read_bits_int_be(4)
            self._io.align_to_byte()
            self.mission_counter = self._io.read_u1()
            self.mission_counter_2 = self._io.read_u1()

        @property
        def vcc_cc2(self):
            """Supply voltage of CC2 (communication controller 2) [V]."""
            if hasattr(self, '_m_vcc_cc2'):
                return self._m_vcc_cc2

            self._m_vcc_cc2 = (((((((((self.vcc_cc2_raw & 128) // 128 * 4) + ((self.vcc_cc2_raw & 64) // 64 * 2)) + ((self.vcc_cc2_raw & 32) // 32 * 1)) + ((self.vcc_cc2_raw & 16) // 16 * 0.5)) + ((self.vcc_cc2_raw & 8) // 8 * 0.25)) + ((self.vcc_cc2_raw & 4) // 4 * 0.125)) + ((self.vcc_cc2_raw & 2) // 2 * 0.0625)) + ((self.vcc_cc2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_vcc_cc2', None)

        @property
        def v_bat2(self):
            """Voltage of the battery 2 [V]."""
            if hasattr(self, '_m_v_bat2'):
                return self._m_v_bat2

            self._m_v_bat2 = (((((((((self.v_bat2_raw & 128) // 128 * 4) + ((self.v_bat2_raw & 64) // 64 * 2)) + ((self.v_bat2_raw & 32) // 32 * 1)) + ((self.v_bat2_raw & 16) // 16 * 0.5)) + ((self.v_bat2_raw & 8) // 8 * 0.25)) + ((self.v_bat2_raw & 4) // 4 * 0.125)) + ((self.v_bat2_raw & 2) // 2 * 0.0625)) + ((self.v_bat2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_bat2', None)

        @property
        def rssi_a(self):
            """Received Signal Strength Indication of TRX module A during the last uplink [dBm]."""
            if hasattr(self, '_m_rssi_a'):
                return self._m_rssi_a

            self._m_rssi_a = (-132.0 + self.rssi_a_raw // 2)
            return getattr(self, '_m_rssi_a', None)

        @property
        def rssi_b(self):
            """Received Signal Strength Indication of TRX module C during the last uplink [dBm]."""
            if hasattr(self, '_m_rssi_b'):
                return self._m_rssi_b

            self._m_rssi_b = (-132.0 + self.rssi_b_raw // 2)
            return getattr(self, '_m_rssi_b', None)

        @property
        def v_pv1(self):
            """Voltage at PV1-bus (Solarbus 1) [V]."""
            if hasattr(self, '_m_v_pv1'):
                return self._m_v_pv1

            self._m_v_pv1 = (((((((((self.v_pv1_raw & 128) // 128 * 4) + ((self.v_pv1_raw & 64) // 64 * 2)) + ((self.v_pv1_raw & 32) // 32 * 1)) + ((self.v_pv1_raw & 16) // 16 * 0.5)) + ((self.v_pv1_raw & 8) // 8 * 0.25)) + ((self.v_pv1_raw & 4) // 4 * 0.125)) + ((self.v_pv1_raw & 2) // 2 * 0.0625)) + ((self.v_pv1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_pv1', None)

        @property
        def v_hv(self):
            """Voltage at the output of the HV supply to the PPTs measured at FET4-2 [V]."""
            if hasattr(self, '_m_v_hv'):
                return self._m_v_hv

            self._m_v_hv = (((((((((self.v_hv_raw & 128) // 128 * 4) + ((self.v_hv_raw & 64) // 64 * 2)) + ((self.v_hv_raw & 32) // 32 * 1)) + ((self.v_hv_raw & 16) // 16 * 0.5)) + ((self.v_hv_raw & 8) // 8 * 0.25)) + ((self.v_hv_raw & 4) // 4 * 0.125)) + ((self.v_hv_raw & 2) // 2 * 0.0625)) + ((self.v_hv_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_hv', None)

        @property
        def v_bat1(self):
            """Voltage of the battery 1 [V]."""
            if hasattr(self, '_m_v_bat1'):
                return self._m_v_bat1

            self._m_v_bat1 = (((((((((self.v_bat1_raw & 128) // 128 * 4) + ((self.v_bat1_raw & 64) // 64 * 2)) + ((self.v_bat1_raw & 32) // 32 * 1)) + ((self.v_bat1_raw & 16) // 16 * 0.5)) + ((self.v_bat1_raw & 8) // 8 * 0.25)) + ((self.v_bat1_raw & 4) // 4 * 0.125)) + ((self.v_bat1_raw & 2) // 2 * 0.0625)) + ((self.v_bat1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_bat1', None)

        @property
        def i_pv1_bat2(self):
            """Current through FET2-1 between PV1-bus and battery 2 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv1_bat2'):
                return self._m_i_pv1_bat2

            self._m_i_pv1_bat2 = (((((((((self.i_pv1_bat2_raw & 128) // 128 * -8) + ((self.i_pv1_bat2_raw & 64) // 64 * 4)) + ((self.i_pv1_bat2_raw & 32) // 32 * 2)) + ((self.i_pv1_bat2_raw & 16) // 16 * 1)) + ((self.i_pv1_bat2_raw & 8) // 8 * 0.5)) + ((self.i_pv1_bat2_raw & 4) // 4 * 0.25)) + ((self.i_pv1_bat2_raw & 2) // 2 * 0.125)) + ((self.i_pv1_bat2_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_bat2', None)

        @property
        def v_3v3_out(self):
            """Voltage at the output of the 3V3 converter [V]."""
            if hasattr(self, '_m_v_3v3_out'):
                return self._m_v_3v3_out

            self._m_v_3v3_out = (((((((((self.v_3v3_out_raw & 128) // 128 * 4) + ((self.v_3v3_out_raw & 64) // 64 * 2)) + ((self.v_3v3_out_raw & 32) // 32 * 1)) + ((self.v_3v3_out_raw & 16) // 16 * 0.5)) + ((self.v_3v3_out_raw & 8) // 8 * 0.25)) + ((self.v_3v3_out_raw & 4) // 4 * 0.125)) + ((self.v_3v3_out_raw & 2) // 2 * 0.0625)) + ((self.v_3v3_out_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_3v3_out', None)

        @property
        def v_3v3_in(self):
            """Voltage at the input of the 3V3 converter measured at FET5-2 [V]."""
            if hasattr(self, '_m_v_3v3_in'):
                return self._m_v_3v3_in

            self._m_v_3v3_in = (((((((((self.v_3v3_in_raw & 128) // 128 * 4) + ((self.v_3v3_in_raw & 64) // 64 * 2)) + ((self.v_3v3_in_raw & 32) // 32 * 1)) + ((self.v_3v3_in_raw & 16) // 16 * 0.5)) + ((self.v_3v3_in_raw & 8) // 8 * 0.25)) + ((self.v_3v3_in_raw & 4) // 4 * 0.125)) + ((self.v_3v3_in_raw & 2) // 2 * 0.0625)) + ((self.v_3v3_in_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_3v3_in', None)

        @property
        def i_pv1_bat1(self):
            """Current through FET1-1 between PV1-bus and battery 1 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv1_bat1'):
                return self._m_i_pv1_bat1

            self._m_i_pv1_bat1 = (((((((((self.i_pv1_bat1_raw & 128) // 128 * -8) + ((self.i_pv1_bat1_raw & 64) // 64 * 4)) + ((self.i_pv1_bat1_raw & 32) // 32 * 2)) + ((self.i_pv1_bat1_raw & 16) // 16 * 1)) + ((self.i_pv1_bat1_raw & 8) // 8 * 0.5)) + ((self.i_pv1_bat1_raw & 4) // 4 * 0.25)) + ((self.i_pv1_bat1_raw & 2) // 2 * 0.125)) + ((self.i_pv1_bat1_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_bat1', None)

        @property
        def i_pv2_3v3(self):
            """Current through FET5-2 between PV2-bus and 3V3 converter [A]."""
            if hasattr(self, '_m_i_pv2_3v3'):
                return self._m_i_pv2_3v3

            self._m_i_pv2_3v3 = (((((((((self.i_pv2_3v3_raw & 128) // 128 * -8) + ((self.i_pv2_3v3_raw & 64) // 64 * 4)) + ((self.i_pv2_3v3_raw & 32) // 32 * 2)) + ((self.i_pv2_3v3_raw & 16) // 16 * 1)) + ((self.i_pv2_3v3_raw & 8) // 8 * 0.5)) + ((self.i_pv2_3v3_raw & 4) // 4 * 0.25)) + ((self.i_pv2_3v3_raw & 2) // 2 * 0.125)) + ((self.i_pv2_3v3_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_3v3', None)

        @property
        def vcc_cc1(self):
            """Supply voltage of CC1 (communication controller 1) [V]."""
            if hasattr(self, '_m_vcc_cc1'):
                return self._m_vcc_cc1

            self._m_vcc_cc1 = (((((((((self.vcc_cc1_raw & 128) // 128 * 4) + ((self.vcc_cc1_raw & 64) // 64 * 2)) + ((self.vcc_cc1_raw & 32) // 32 * 1)) + ((self.vcc_cc1_raw & 16) // 16 * 0.5)) + ((self.vcc_cc1_raw & 8) // 8 * 0.25)) + ((self.vcc_cc1_raw & 4) // 4 * 0.125)) + ((self.vcc_cc1_raw & 2) // 2 * 0.0625)) + ((self.vcc_cc1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_vcc_cc1', None)

        @property
        def v_5v_out(self):
            """Voltage at the output of the 5V converter [V]."""
            if hasattr(self, '_m_v_5v_out'):
                return self._m_v_5v_out

            self._m_v_5v_out = (((((((((self.v_5v_out_raw & 128) // 128 * 4) + ((self.v_5v_out_raw & 64) // 64 * 2)) + ((self.v_5v_out_raw & 32) // 32 * 1)) + ((self.v_5v_out_raw & 16) // 16 * 0.5)) + ((self.v_5v_out_raw & 8) // 8 * 0.25)) + ((self.v_5v_out_raw & 4) // 4 * 0.125)) + ((self.v_5v_out_raw & 2) // 2 * 0.0625)) + ((self.v_5v_out_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_5v_out', None)

        @property
        def v_5v_in(self):
            """Voltage at the input of the 5V converter measured at FET3-1 [V]."""
            if hasattr(self, '_m_v_5v_in'):
                return self._m_v_5v_in

            self._m_v_5v_in = (((((((((self.v_5v_in_raw & 128) // 128 * 4) + ((self.v_5v_in_raw & 64) // 64 * 2)) + ((self.v_5v_in_raw & 32) // 32 * 1)) + ((self.v_5v_in_raw & 16) // 16 * 0.5)) + ((self.v_5v_in_raw & 8) // 8 * 0.25)) + ((self.v_5v_in_raw & 4) // 4 * 0.125)) + ((self.v_5v_in_raw & 2) // 2 * 0.0625)) + ((self.v_5v_in_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_5v_in', None)

        @property
        def i_pv1_5v(self):
            """Current through FET3-1 between PV1-bus and 5V converter [A]."""
            if hasattr(self, '_m_i_pv1_5v'):
                return self._m_i_pv1_5v

            self._m_i_pv1_5v = (((((((((self.i_pv1_5v_raw & 128) // 128 * -8) + ((self.i_pv1_5v_raw & 64) // 64 * 4)) + ((self.i_pv1_5v_raw & 32) // 32 * 2)) + ((self.i_pv1_5v_raw & 16) // 16 * 1)) + ((self.i_pv1_5v_raw & 8) // 8 * 0.5)) + ((self.i_pv1_5v_raw & 4) // 4 * 0.25)) + ((self.i_pv1_5v_raw & 2) // 2 * 0.125)) + ((self.i_pv1_5v_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_5v', None)

        @property
        def i_pv2_bat2(self):
            """Current through FET2-2 between PV2-bus and battery 2 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv2_bat2'):
                return self._m_i_pv2_bat2

            self._m_i_pv2_bat2 = (((((((((self.i_pv2_bat2_raw & 128) // 128 * -8) + ((self.i_pv2_bat2_raw & 64) // 64 * 4)) + ((self.i_pv2_bat2_raw & 32) // 32 * 2)) + ((self.i_pv2_bat2_raw & 16) // 16 * 1)) + ((self.i_pv2_bat2_raw & 8) // 8 * 0.5)) + ((self.i_pv2_bat2_raw & 4) // 4 * 0.25)) + ((self.i_pv2_bat2_raw & 2) // 2 * 0.125)) + ((self.i_pv2_bat2_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_bat2', None)

        @property
        def i_pv2_bat1(self):
            """Current through FET1-2 between PV2-bus and battery 1 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv2_bat1'):
                return self._m_i_pv2_bat1

            self._m_i_pv2_bat1 = (((((((((self.i_pv2_bat1_raw & 128) // 128 * -8) + ((self.i_pv2_bat1_raw & 64) // 64 * 4)) + ((self.i_pv2_bat1_raw & 32) // 32 * 2)) + ((self.i_pv2_bat1_raw & 16) // 16 * 1)) + ((self.i_pv2_bat1_raw & 8) // 8 * 0.5)) + ((self.i_pv2_bat1_raw & 4) // 4 * 0.25)) + ((self.i_pv2_bat1_raw & 2) // 2 * 0.125)) + ((self.i_pv2_bat1_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_bat1', None)

        @property
        def i_pv1_3v3(self):
            """Current through FET5-1 between PV1-bus and 3V3 converter [A]."""
            if hasattr(self, '_m_i_pv1_3v3'):
                return self._m_i_pv1_3v3

            self._m_i_pv1_3v3 = (((((((((self.i_pv1_3v3_raw & 128) // 128 * -8) + ((self.i_pv1_3v3_raw & 64) // 64 * 4)) + ((self.i_pv1_3v3_raw & 32) // 32 * 2)) + ((self.i_pv1_3v3_raw & 16) // 16 * 1)) + ((self.i_pv1_3v3_raw & 8) // 8 * 0.5)) + ((self.i_pv1_3v3_raw & 4) // 4 * 0.25)) + ((self.i_pv1_3v3_raw & 2) // 2 * 0.125)) + ((self.i_pv1_3v3_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_3v3', None)

        @property
        def v_pv2(self):
            """Voltage at PV2-bus (Solarbus 1) [V]."""
            if hasattr(self, '_m_v_pv2'):
                return self._m_v_pv2

            self._m_v_pv2 = (((((((((self.v_pv2_raw & 128) // 128 * 4) + ((self.v_pv2_raw & 64) // 64 * 2)) + ((self.v_pv2_raw & 32) // 32 * 1)) + ((self.v_pv2_raw & 16) // 16 * 0.5)) + ((self.v_pv2_raw & 8) // 8 * 0.25)) + ((self.v_pv2_raw & 4) // 4 * 0.125)) + ((self.v_pv2_raw & 2) // 2 * 0.0625)) + ((self.v_pv2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_pv2', None)

        @property
        def i_pv2_5v(self):
            """Current through FET3-2 between PV2-bus and 5V converter [A]."""
            if hasattr(self, '_m_i_pv2_5v'):
                return self._m_i_pv2_5v

            self._m_i_pv2_5v = (((((((((self.i_pv2_5v_raw & 128) // 128 * -8) + ((self.i_pv2_5v_raw & 64) // 64 * 4)) + ((self.i_pv2_5v_raw & 32) // 32 * 2)) + ((self.i_pv2_5v_raw & 16) // 16 * 1)) + ((self.i_pv2_5v_raw & 8) // 8 * 0.5)) + ((self.i_pv2_5v_raw & 4) // 4 * 0.25)) + ((self.i_pv2_5v_raw & 2) // 2 * 0.125)) + ((self.i_pv2_5v_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_5v', None)


    class ABeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.call = (self._io.read_bytes(6)).decode(u"ASCII")
            self.ant_temp_raw = self._io.read_u2le()
            self.ant_deploy_status_mb = self._io.read_u1()
            self.ant_deploy_status_lb = self._io.read_u1()
            self.temp_comp_hw = self._io.read_u1()
            self.padding_a1 = self._io.read_bytes(11)
            self.rbf_state = self._io.read_u1()
            self.sid_a = self._io.read_u1()
            self.stacie_version = self._io.read_u1()

        @property
        def ant_temp(self):
            """Antenna temperature [mA]."""
            if hasattr(self, '_m_ant_temp'):
                return self._m_ant_temp

            self._m_ant_temp = (((3.3 / 1023) * (((self.ant_temp_raw & 255) * 256) + (self.ant_temp_raw & 65280))) * 1000)
            return getattr(self, '_m_ant_temp', None)


    class SBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.call = (self._io.read_bytes(6)).decode(u"ASCII")
            self.usp_raw = self._io.read_u2le()
            self.trx_temp = self._io.read_s1()
            self.rssi_idle_raw = self._io.read_u1()
            self.rssi_rx_raw = self._io.read_u1()
            self.antenna_deployment = self._io.read_bits_int_be(8)
            self._io.align_to_byte()
            self.stacie_op = self._io.read_u1()
            self.temp_compensation = self._io.read_u1()
            self.reset_counter = self._io.read_u2le()
            self.uplink_error = self._io.read_u1()
            self.obc_sent_packet_counter_between_s_beacons = self._io.read_u1()
            self.beacon_interval = self._io.read_u2le()
            self.primary_carrier_raw = self._io.read_u2le()
            self.secondary_carrier_raw = self._io.read_u2le()
            self.used_carrier_raw = self._io.read_u2le()
            self.temperature_compensation_carrier_raw = self._io.read_u2le()
            self.sid_s = self._io.read_u1()
            self.tx_sel_reason = self._io.read_u1()
            self.reason_remote = self._io.read_u1()
            self.s_time_raw = self._io.read_u4le()
            self.gs_cmd_counter = self._io.read_u1()
            self.beacon_count = self._io.read_u1()
            self.prim_freq_start = self._io.read_u2le()
            self.sec_freq_start = self._io.read_u2le()

        @property
        def used_carrier(self):
            """Used carrier frequency for this S-beacon [Hz]."""
            if hasattr(self, '_m_used_carrier'):
                return self._m_used_carrier

            self._m_used_carrier = ((((self.used_carrier_raw * 156.25) / 1000) + 430000) * 100)
            return getattr(self, '_m_used_carrier', None)

        @property
        def s_time(self):
            """STACIE uptime since last reset [s]."""
            if hasattr(self, '_m_s_time'):
                return self._m_s_time

            self._m_s_time = self.s_time_raw // 1000
            return getattr(self, '_m_s_time', None)

        @property
        def usp(self):
            """Supply voltage of STACIE [V]."""
            if hasattr(self, '_m_usp'):
                return self._m_usp

            self._m_usp = ((((self.usp_raw & 255) * 256) + (self.usp_raw & 65280)) / ((1023 * 2) * 3.3))
            return getattr(self, '_m_usp', None)

        @property
        def rssi_idle(self):
            """Received Signal Strength Indication of TRX module without TX [dBm]."""
            if hasattr(self, '_m_rssi_idle'):
                return self._m_rssi_idle

            self._m_rssi_idle = ((((((((((self.rssi_idle_raw & 128) // 128 * 64) + ((self.rssi_idle_raw & 64) // 64 * 32)) + ((self.rssi_idle_raw & 32) // 32 * 16)) + ((self.rssi_idle_raw & 16) // 16 * 8)) + ((self.rssi_idle_raw & 8) // 8 * 4)) + ((self.rssi_idle_raw & 4) // 4 * 2)) + ((self.rssi_idle_raw & 2) // 2 * 1)) + ((self.rssi_idle_raw & 1) // 1 * 0.5)) - 132.0)
            return getattr(self, '_m_rssi_idle', None)

        @property
        def primary_carrier(self):
            """Carrier Frequency of the primary RX frequency [Hz]."""
            if hasattr(self, '_m_primary_carrier'):
                return self._m_primary_carrier

            self._m_primary_carrier = ((((self.primary_carrier_raw * 156.25) / 1000) + 430000) * 100)
            return getattr(self, '_m_primary_carrier', None)

        @property
        def rssi_rx(self):
            """Received Signal Strength Indication of TRX module during RX [dBm]."""
            if hasattr(self, '_m_rssi_rx'):
                return self._m_rssi_rx

            self._m_rssi_rx = ((((((((((self.rssi_rx_raw & 128) // 128 * 64) + ((self.rssi_rx_raw & 64) // 64 * 32)) + ((self.rssi_rx_raw & 32) // 32 * 16)) + ((self.rssi_rx_raw & 16) // 16 * 8)) + ((self.rssi_rx_raw & 8) // 8 * 4)) + ((self.rssi_rx_raw & 4) // 4 * 2)) + ((self.rssi_rx_raw & 2) // 2 * 1)) + ((self.rssi_rx_raw & 1) // 1 * 0.5)) - 132.0)
            return getattr(self, '_m_rssi_rx', None)

        @property
        def temperature_compensation_carrier(self):
            """Frequency compensated carrier [Hz]."""
            if hasattr(self, '_m_temperature_compensation_carrier'):
                return self._m_temperature_compensation_carrier

            self._m_temperature_compensation_carrier = ((((self.temperature_compensation_carrier_raw * 156.25) / 1000) + 430000) * 100)
            return getattr(self, '_m_temperature_compensation_carrier', None)

        @property
        def secondary_carrier(self):
            """Carrier Frequency of the secondary RX frequency [Hz]."""
            if hasattr(self, '_m_secondary_carrier'):
                return self._m_secondary_carrier

            self._m_secondary_carrier = ((((self.secondary_carrier_raw * 156.25) / 1000) + 430000) * 100)
            return getattr(self, '_m_secondary_carrier', None)


    class EBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.call = (self._io.read_bytes(6)).decode(u"ASCII")
            self.i_pv2_5v_raw = self._io.read_s1()
            self.i_pv1_5v_raw = self._io.read_s1()
            self.v_pv2_raw = self._io.read_u1()
            self.v_5v_in_raw = self._io.read_u1()
            self.i_pv1_3v3_raw = self._io.read_s1()
            self.i_pv2_3v3_raw = self._io.read_s1()
            self.v_pv1_raw = self._io.read_u1()
            self.v_3v3_in_raw = self._io.read_u1()
            self.temp_bat1sw = self._io.read_s1()
            self.temp_5v = self._io.read_s1()
            self.i_pv1_hv_raw = self._io.read_s1()
            self.i_pv2_hv_raw = self._io.read_s1()
            self.v_3v3_out_raw = self._io.read_u1()
            self.v_hv_raw = self._io.read_u1()
            self.i_pv2_bat1_raw = self._io.read_s1()
            self.i_pv1_bat1_raw = self._io.read_s1()
            self.v_5v_out_raw = self._io.read_u1()
            self.v_bat1_raw = self._io.read_u1()
            self.i_pv2_bat2_raw = self._io.read_s1()
            self.i_pv1_bat2_raw = self._io.read_s1()
            self.eps_version = self._io.read_u1()
            self.sid_e = self._io.read_u1()
            self.v_bat2_raw = self._io.read_u1()
            self.temp_bat1 = self._io.read_s1()
            self.temp_bat2 = self._io.read_s1()
            self.status_1_3v3_1 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_2 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_3 = self._io.read_bits_int_be(1) != 0
            self.status_1_3v3_bu = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_1 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_2 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_3 = self._io.read_bits_int_be(1) != 0
            self.status_1_5v_4 = self._io.read_bits_int_be(1) != 0
            self.status_2_power_low = self._io.read_bits_int_be(1) != 0
            self.status_2_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_2_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_2_3v3 = self._io.read_bits_int_be(1) != 0
            self.status_2_5v = self._io.read_bits_int_be(1) != 0
            self.status_2_mode = self._io.read_bits_int_be(3)
            self.status_3_3v3_burst = self._io.read_bits_int_be(1) != 0
            self.status_3_5v_burst = self._io.read_bits_int_be(1) != 0
            self.status_3_bat1_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_3_bat2_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_3_temp_warning = self._io.read_bits_int_be(1) != 0
            self.status_3_cc1 = self._io.read_bits_int_be(1) != 0
            self.status_3_cc2 = self._io.read_bits_int_be(1) != 0
            self.status_3_rbf = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.padding_e1 = self._io.read_bytes(1)
            self.s_beacon_count = self._io.read_u1()
            self.reboot_mc = self._io.read_u1()
            self.reboot_cc1 = self._io.read_u1()
            self.reboot_cc2 = self._io.read_u1()
            self.vcc_cc1_raw = self._io.read_u1()
            self.temp_cc1 = self._io.read_s1()
            self.vcc_cc2_raw = self._io.read_u1()
            self.temp_cc2 = self._io.read_s1()
            self.status_cc1_mode = self._io.read_bits_int_be(2)
            self.status_cc1_mc_timeout = self._io.read_bits_int_be(1) != 0
            self.status_cc1_tbd = self._io.read_bits_int_be(1) != 0
            self.status_cc1_en_i2c = self._io.read_bits_int_be(1) != 0
            self.status_cc1_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_cc1_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_cc1_3v3_backup = self._io.read_bits_int_be(1) != 0
            self.status_cc2_mode = self._io.read_bits_int_be(2)
            self.status_cc2_mc_timeout = self._io.read_bits_int_be(1) != 0
            self.status_cc2_tbd = self._io.read_bits_int_be(1) != 0
            self.status_cc2_en_i2c = self._io.read_bits_int_be(1) != 0
            self.status_cc2_bat1_pv1 = self._io.read_bits_int_be(1) != 0
            self.status_cc2_bat2_pv2 = self._io.read_bits_int_be(1) != 0
            self.status_cc2_3v3_backup = self._io.read_bits_int_be(1) != 0

        @property
        def vcc_cc2(self):
            """Supply voltage of CC (communication controller) [V]."""
            if hasattr(self, '_m_vcc_cc2'):
                return self._m_vcc_cc2

            self._m_vcc_cc2 = (((((((((self.vcc_cc2_raw & 128) // 128 * 4) + ((self.vcc_cc2_raw & 64) // 64 * 2)) + ((self.vcc_cc2_raw & 32) // 32 * 1)) + ((self.vcc_cc2_raw & 16) // 16 * 0.5)) + ((self.vcc_cc2_raw & 8) // 8 * 0.25)) + ((self.vcc_cc2_raw & 4) // 4 * 0.125)) + ((self.vcc_cc2_raw & 2) // 2 * 0.0625)) + ((self.vcc_cc2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_vcc_cc2', None)

        @property
        def v_bat2(self):
            """Voltage of the battery 2."""
            if hasattr(self, '_m_v_bat2'):
                return self._m_v_bat2

            self._m_v_bat2 = (((((((((self.v_bat2_raw & 128) // 128 * 4) + ((self.v_bat2_raw & 64) // 64 * 2)) + ((self.v_bat2_raw & 32) // 32 * 1)) + ((self.v_bat2_raw & 16) // 16 * 0.5)) + ((self.v_bat2_raw & 8) // 8 * 0.25)) + ((self.v_bat2_raw & 4) // 4 * 0.125)) + ((self.v_bat2_raw & 2) // 2 * 0.0625)) + ((self.v_bat2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_bat2', None)

        @property
        def v_pv1(self):
            """Voltage at PV1-bus (Solarbus 1) [V]."""
            if hasattr(self, '_m_v_pv1'):
                return self._m_v_pv1

            self._m_v_pv1 = (((((((((self.v_pv1_raw & 128) // 128 * 4) + ((self.v_pv1_raw & 64) // 64 * 2)) + ((self.v_pv1_raw & 32) // 32 * 1)) + ((self.v_pv1_raw & 16) // 16 * 0.5)) + ((self.v_pv1_raw & 8) // 8 * 0.25)) + ((self.v_pv1_raw & 4) // 4 * 0.125)) + ((self.v_pv1_raw & 2) // 2 * 0.0625)) + ((self.v_pv1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_pv1', None)

        @property
        def v_hv(self):
            """Voltage at the output of the HV supply to the PPTs measured at FET4-2 [V]."""
            if hasattr(self, '_m_v_hv'):
                return self._m_v_hv

            self._m_v_hv = (((((((((self.v_hv_raw & 128) // 128 * 4) + ((self.v_hv_raw & 64) // 64 * 2)) + ((self.v_hv_raw & 32) // 32 * 1)) + ((self.v_hv_raw & 16) // 16 * 0.5)) + ((self.v_hv_raw & 8) // 8 * 0.25)) + ((self.v_hv_raw & 4) // 4 * 0.125)) + ((self.v_hv_raw & 2) // 2 * 0.0625)) + ((self.v_hv_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_hv', None)

        @property
        def v_bat1(self):
            """Voltage of the battery 1 [V]."""
            if hasattr(self, '_m_v_bat1'):
                return self._m_v_bat1

            self._m_v_bat1 = (((((((((self.v_bat1_raw & 128) // 128 * 4) + ((self.v_bat1_raw & 64) // 64 * 2)) + ((self.v_bat1_raw & 32) // 32 * 1)) + ((self.v_bat1_raw & 16) // 16 * 0.5)) + ((self.v_bat1_raw & 8) // 8 * 0.25)) + ((self.v_bat1_raw & 4) // 4 * 0.125)) + ((self.v_bat1_raw & 2) // 2 * 0.0625)) + ((self.v_bat1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_bat1', None)

        @property
        def i_pv1_bat2(self):
            """Current through FET2-1 between PV1-bus and battery 2 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv1_bat2'):
                return self._m_i_pv1_bat2

            self._m_i_pv1_bat2 = (((((((((self.i_pv1_bat2_raw & 128) // 128 * -8) + ((self.i_pv1_bat2_raw & 64) // 64 * 4)) + ((self.i_pv1_bat2_raw & 32) // 32 * 2)) + ((self.i_pv1_bat2_raw & 16) // 16 * 1)) + ((self.i_pv1_bat2_raw & 8) // 8 * 0.5)) + ((self.i_pv1_bat2_raw & 4) // 4 * 0.25)) + ((self.i_pv1_bat2_raw & 2) // 2 * 0.125)) + ((self.i_pv1_bat2_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_bat2', None)

        @property
        def v_3v3_out(self):
            """Voltage at the output of the 3V3 converter [V]."""
            if hasattr(self, '_m_v_3v3_out'):
                return self._m_v_3v3_out

            self._m_v_3v3_out = (((((((((self.v_3v3_out_raw & 128) // 128 * 4) + ((self.v_3v3_out_raw & 64) // 64 * 2)) + ((self.v_3v3_out_raw & 32) // 32 * 1)) + ((self.v_3v3_out_raw & 16) // 16 * 0.5)) + ((self.v_3v3_out_raw & 8) // 8 * 0.25)) + ((self.v_3v3_out_raw & 4) // 4 * 0.125)) + ((self.v_3v3_out_raw & 2) // 2 * 0.0625)) + ((self.v_3v3_out_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_3v3_out', None)

        @property
        def v_3v3_in(self):
            """Voltage at the input of the 3V3 converter measured at FET5-2 [V]."""
            if hasattr(self, '_m_v_3v3_in'):
                return self._m_v_3v3_in

            self._m_v_3v3_in = (((((((((self.v_3v3_in_raw & 128) // 128 * 4) + ((self.v_3v3_in_raw & 64) // 64 * 2)) + ((self.v_3v3_in_raw & 32) // 32 * 1)) + ((self.v_3v3_in_raw & 16) // 16 * 0.5)) + ((self.v_3v3_in_raw & 8) // 8 * 0.25)) + ((self.v_3v3_in_raw & 4) // 4 * 0.125)) + ((self.v_3v3_in_raw & 2) // 2 * 0.0625)) + ((self.v_3v3_in_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_3v3_in', None)

        @property
        def i_pv1_bat1(self):
            """Current through FET1-1 between PV1-bus and battery 1 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv1_bat1'):
                return self._m_i_pv1_bat1

            self._m_i_pv1_bat1 = (((((((((self.i_pv1_bat1_raw & 128) // 128 * -8) + ((self.i_pv1_bat1_raw & 64) // 64 * 4)) + ((self.i_pv1_bat1_raw & 32) // 32 * 2)) + ((self.i_pv1_bat1_raw & 16) // 16 * 1)) + ((self.i_pv1_bat1_raw & 8) // 8 * 0.5)) + ((self.i_pv1_bat1_raw & 4) // 4 * 0.25)) + ((self.i_pv1_bat1_raw & 2) // 2 * 0.125)) + ((self.i_pv1_bat1_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_bat1', None)

        @property
        def i_pv2_3v3(self):
            """Current through FET5-2 between PV2-bus and 3V3 converter."""
            if hasattr(self, '_m_i_pv2_3v3'):
                return self._m_i_pv2_3v3

            self._m_i_pv2_3v3 = (((((((((self.i_pv2_3v3_raw & 128) // 128 * -8) + ((self.i_pv2_3v3_raw & 64) // 64 * 4)) + ((self.i_pv2_3v3_raw & 32) // 32 * 2)) + ((self.i_pv2_3v3_raw & 16) // 16 * 1)) + ((self.i_pv2_3v3_raw & 8) // 8 * 0.5)) + ((self.i_pv2_3v3_raw & 4) // 4 * 0.25)) + ((self.i_pv2_3v3_raw & 2) // 2 * 0.125)) + ((self.i_pv2_3v3_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_3v3', None)

        @property
        def vcc_cc1(self):
            """Supply voltage of CC1 (communication controller 1) [V]."""
            if hasattr(self, '_m_vcc_cc1'):
                return self._m_vcc_cc1

            self._m_vcc_cc1 = (((((((((self.vcc_cc1_raw & 128) // 128 * 4) + ((self.vcc_cc1_raw & 64) // 64 * 2)) + ((self.vcc_cc1_raw & 32) // 32 * 1)) + ((self.vcc_cc1_raw & 16) // 16 * 0.5)) + ((self.vcc_cc1_raw & 8) // 8 * 0.25)) + ((self.vcc_cc1_raw & 4) // 4 * 0.125)) + ((self.vcc_cc1_raw & 2) // 2 * 0.0625)) + ((self.vcc_cc1_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_vcc_cc1', None)

        @property
        def v_5v_out(self):
            """Voltage at the output of the 5V converter [V]."""
            if hasattr(self, '_m_v_5v_out'):
                return self._m_v_5v_out

            self._m_v_5v_out = (((((((((self.v_5v_out_raw & 128) // 128 * 4) + ((self.v_5v_out_raw & 64) // 64 * 2)) + ((self.v_5v_out_raw & 32) // 32 * 1)) + ((self.v_5v_out_raw & 16) // 16 * 0.5)) + ((self.v_5v_out_raw & 8) // 8 * 0.25)) + ((self.v_5v_out_raw & 4) // 4 * 0.125)) + ((self.v_5v_out_raw & 2) // 2 * 0.0625)) + ((self.v_5v_out_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_5v_out', None)

        @property
        def v_5v_in(self):
            """Voltage at input 5V converter measured at FET3-1 [V]."""
            if hasattr(self, '_m_v_5v_in'):
                return self._m_v_5v_in

            self._m_v_5v_in = (((((((((self.v_5v_in_raw & 128) // 128 * 4) + ((self.v_5v_in_raw & 64) // 64 * 2)) + ((self.v_5v_in_raw & 32) // 32 * 1)) + ((self.v_5v_in_raw & 16) // 16 * 0.5)) + ((self.v_5v_in_raw & 8) // 8 * 0.25)) + ((self.v_5v_in_raw & 4) // 4 * 0.125)) + ((self.v_5v_in_raw & 2) // 2 * 0.0625)) + ((self.v_5v_in_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_5v_in', None)

        @property
        def i_pv1_5v(self):
            """Current through FET3-1 between PV1-bus and 5V converter."""
            if hasattr(self, '_m_i_pv1_5v'):
                return self._m_i_pv1_5v

            self._m_i_pv1_5v = (((((((((self.i_pv1_5v_raw & 128) // 128 * -8) + ((self.i_pv1_5v_raw & 64) // 64 * 4)) + ((self.i_pv1_5v_raw & 32) // 32 * 2)) + ((self.i_pv1_5v_raw & 16) // 16 * 1)) + ((self.i_pv1_5v_raw & 8) // 8 * 0.5)) + ((self.i_pv1_5v_raw & 4) // 4 * 0.25)) + ((self.i_pv1_5v_raw & 2) // 2 * 0.125)) + ((self.i_pv1_5v_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_5v', None)

        @property
        def i_pv2_bat2(self):
            """Current through FET2-2 between PV2-bus and battery 2 [A]
            + charging, - discharging  
            """
            if hasattr(self, '_m_i_pv2_bat2'):
                return self._m_i_pv2_bat2

            self._m_i_pv2_bat2 = (((((((((self.i_pv2_bat2_raw & 128) // 128 * -8) + ((self.i_pv2_bat2_raw & 64) // 64 * 4)) + ((self.i_pv2_bat2_raw & 32) // 32 * 2)) + ((self.i_pv2_bat2_raw & 16) // 16 * 1)) + ((self.i_pv2_bat2_raw & 8) // 8 * 0.5)) + ((self.i_pv2_bat2_raw & 4) // 4 * 0.25)) + ((self.i_pv2_bat2_raw & 2) // 2 * 0.125)) + ((self.i_pv2_bat2_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_bat2', None)

        @property
        def i_pv2_bat1(self):
            """Current through FET1-2 between PV2-bus and battery 1 [A]
            + charging, - discharging
            """
            if hasattr(self, '_m_i_pv2_bat1'):
                return self._m_i_pv2_bat1

            self._m_i_pv2_bat1 = (((((((((self.i_pv2_bat1_raw & 128) // 128 * -8) + ((self.i_pv2_bat1_raw & 64) // 64 * 4)) + ((self.i_pv2_bat1_raw & 32) // 32 * 2)) + ((self.i_pv2_bat1_raw & 16) // 16 * 1)) + ((self.i_pv2_bat1_raw & 8) // 8 * 0.5)) + ((self.i_pv2_bat1_raw & 4) // 4 * 0.25)) + ((self.i_pv2_bat1_raw & 2) // 2 * 0.125)) + ((self.i_pv2_bat1_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_bat1', None)

        @property
        def i_pv1_3v3(self):
            """Current through FET5-1 between PV1-bus and 3V3 converter [A]."""
            if hasattr(self, '_m_i_pv1_3v3'):
                return self._m_i_pv1_3v3

            self._m_i_pv1_3v3 = (((((((((self.i_pv1_3v3_raw & 128) // 128 * -8) + ((self.i_pv1_3v3_raw & 64) // 64 * 4)) + ((self.i_pv1_3v3_raw & 32) // 32 * 2)) + ((self.i_pv1_3v3_raw & 16) // 16 * 1)) + ((self.i_pv1_3v3_raw & 8) // 8 * 0.5)) + ((self.i_pv1_3v3_raw & 4) // 4 * 0.25)) + ((self.i_pv1_3v3_raw & 2) // 2 * 0.125)) + ((self.i_pv1_3v3_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv1_3v3', None)

        @property
        def v_pv2(self):
            """Voltage at PV2-bus (Solarbus 2) [V]."""
            if hasattr(self, '_m_v_pv2'):
                return self._m_v_pv2

            self._m_v_pv2 = (((((((((self.v_pv2_raw & 128) // 128 * 4) + ((self.v_pv2_raw & 64) // 64 * 2)) + ((self.v_pv2_raw & 32) // 32 * 1)) + ((self.v_pv2_raw & 16) // 16 * 0.5)) + ((self.v_pv2_raw & 8) // 8 * 0.25)) + ((self.v_pv2_raw & 4) // 4 * 0.125)) + ((self.v_pv2_raw & 2) // 2 * 0.0625)) + ((self.v_pv2_raw & 1) // 1 * 0.03125))
            return getattr(self, '_m_v_pv2', None)

        @property
        def i_pv2_5v(self):
            """Current through FET3-2 between PV2-bus and 5V converter [A]."""
            if hasattr(self, '_m_i_pv2_5v'):
                return self._m_i_pv2_5v

            self._m_i_pv2_5v = (((((((((self.i_pv2_5v_raw & 128) // 128 * -8) + ((self.i_pv2_5v_raw & 64) // 64 * 4)) + ((self.i_pv2_5v_raw & 32) // 32 * 2)) + ((self.i_pv2_5v_raw & 16) // 16 * 1)) + ((self.i_pv2_5v_raw & 8) // 8 * 0.5)) + ((self.i_pv2_5v_raw & 4) // 4 * 0.25)) + ((self.i_pv2_5v_raw & 2) // 2 * 0.125)) + ((self.i_pv2_5v_raw & 1) // 1 * 0.0625))
            return getattr(self, '_m_i_pv2_5v', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Astrocast(KaitaiStruct):
    """:field dest_callsign: ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_header.src_callsign_raw.callsign_ror.callsign
    :field utc_time_of_position_h: gprmc.utc_time_of_position_h
    :field utc_time_of_position_m: gprmc.utc_time_of_position_m
    :field utc_time_of_position_s: gprmc.utc_time_of_position_s
    :field status: gprmc.status
    :field latitude: gprmc.latitude
    :field latitude_n_or_s: gprmc.latitude_n_or_s
    :field longitude: gprmc.longitude
    :field longitude_e_or_w: gprmc.longitude_e_or_w
    :field ground_speed_knots: gprmc.ground_speed_knots
    :field course_over_ground: gprmc.course_over_ground
    :field date_yy: gprmc.date_yy
    :field date_mm: gprmc.date_mm
    :field date_dd: gprmc.date_dd
    :field magnetic_variation: gprmc.magnetic_variation
    :field magnetic_variation_e_or_w: gprmc.magnetic_variation_e_or_w
    :field timestamp_seconds: tlm.timestamp_seconds
    :field voltage_v: tlm.voltage_v
    :field current_ma: tlm.current_ma
    :field temperature_c: tlm.temperature_c
    :field rssi_dbm: tlm.rssi_dbm
    :field afc_hz: tlm.afc_hz
    :field link_settings: tlm.link_settings
    :field transfer_layer_idle_frames: tlm.transfer_layer_idle_frames
    :field downlink_reed_solomon: tlm.downlink_reed_solomon
    :field downlink_randomizer: tlm.downlink_randomizer
    :field downlink_convolutional: tlm.downlink_convolutional
    :field uplink_bch: tlm.uplink_bch
    :field uplink_derandomizer: tlm.uplink_derandomizer
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_header = Astrocast.Ax25Header(self._io, self, self._root)
        self.gprmc = Astrocast.Gprmc(self._io, self, self._root)
        self.tlm = Astrocast.Tlm(self._io, self, self._root)

    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Astrocast.CallsignRaw(self._io, self, self._root)
            self.dest_ssid = self._io.read_u1()
            self.src_callsign_raw = Astrocast.CallsignRaw(self._io, self, self._root)
            self.src_ssid = self._io.read_u1()
            self.ctl_pid = self._io.read_u2be()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Astrocast.Callsign(_io__raw_callsign_ror, self, self._root)


    class Tlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.hk_prefix = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.skip_0x = self._io.read_u2be()
            self.timestamp_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.voltage_before_dot_raw = (self._io.read_bytes_term(46, False, True, True)).decode(u"UTF-8")
            self.voltage_after_dot_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.current_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.temperature_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.rssi_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.afc_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.skipping_0x = self._io.read_u2be()
            self.link_settings_raw = (self._io.read_bytes(2)).decode(u"UTF-8")

        @property
        def current_ma(self):
            if hasattr(self, '_m_current_ma'):
                return self._m_current_ma

            self._m_current_ma = int(self.current_raw)
            return getattr(self, '_m_current_ma', None)

        @property
        def uplink_bch(self):
            if hasattr(self, '_m_uplink_bch'):
                return self._m_uplink_bch

            self._m_uplink_bch = (True if ((self.link_settings & 8) >> 3) == 1 else False)
            return getattr(self, '_m_uplink_bch', None)

        @property
        def downlink_randomizer(self):
            if hasattr(self, '_m_downlink_randomizer'):
                return self._m_downlink_randomizer

            self._m_downlink_randomizer = (True if ((self.link_settings & 32) >> 5) == 1 else False)
            return getattr(self, '_m_downlink_randomizer', None)

        @property
        def voltage_after_dot(self):
            if hasattr(self, '_m_voltage_after_dot'):
                return self._m_voltage_after_dot

            self._m_voltage_after_dot = int(self.voltage_after_dot_raw)
            return getattr(self, '_m_voltage_after_dot', None)

        @property
        def rssi_dbm(self):
            if hasattr(self, '_m_rssi_dbm'):
                return self._m_rssi_dbm

            self._m_rssi_dbm = int(self.rssi_raw)
            return getattr(self, '_m_rssi_dbm', None)

        @property
        def voltage_v(self):
            if hasattr(self, '_m_voltage_v'):
                return self._m_voltage_v

            self._m_voltage_v = ((self.voltage_after_dot * 0.001) + self.voltage_before_dot)
            return getattr(self, '_m_voltage_v', None)

        @property
        def downlink_convolutional(self):
            if hasattr(self, '_m_downlink_convolutional'):
                return self._m_downlink_convolutional

            self._m_downlink_convolutional = (True if ((self.link_settings & 16) >> 4) == 1 else False)
            return getattr(self, '_m_downlink_convolutional', None)

        @property
        def transfer_layer_idle_frames(self):
            if hasattr(self, '_m_transfer_layer_idle_frames'):
                return self._m_transfer_layer_idle_frames

            self._m_transfer_layer_idle_frames = (True if ((self.link_settings & 128) >> 7) == 1 else False)
            return getattr(self, '_m_transfer_layer_idle_frames', None)

        @property
        def downlink_reed_solomon(self):
            if hasattr(self, '_m_downlink_reed_solomon'):
                return self._m_downlink_reed_solomon

            self._m_downlink_reed_solomon = (True if ((self.link_settings & 64) >> 6) == 1 else False)
            return getattr(self, '_m_downlink_reed_solomon', None)

        @property
        def timestamp_ticks(self):
            if hasattr(self, '_m_timestamp_ticks'):
                return self._m_timestamp_ticks

            self._m_timestamp_ticks = int(self.timestamp_raw, 16)
            return getattr(self, '_m_timestamp_ticks', None)

        @property
        def uplink_derandomizer(self):
            if hasattr(self, '_m_uplink_derandomizer'):
                return self._m_uplink_derandomizer

            self._m_uplink_derandomizer = (True if ((self.link_settings & 4) >> 2) == 1 else False)
            return getattr(self, '_m_uplink_derandomizer', None)

        @property
        def afc_hz(self):
            if hasattr(self, '_m_afc_hz'):
                return self._m_afc_hz

            self._m_afc_hz = int(self.afc_raw)
            return getattr(self, '_m_afc_hz', None)

        @property
        def timestamp_seconds(self):
            if hasattr(self, '_m_timestamp_seconds'):
                return self._m_timestamp_seconds

            self._m_timestamp_seconds = self.timestamp_ticks // 65536
            return getattr(self, '_m_timestamp_seconds', None)

        @property
        def temperature_c(self):
            if hasattr(self, '_m_temperature_c'):
                return self._m_temperature_c

            self._m_temperature_c = int(self.temperature_raw)
            return getattr(self, '_m_temperature_c', None)

        @property
        def link_settings(self):
            if hasattr(self, '_m_link_settings'):
                return self._m_link_settings

            self._m_link_settings = int(self.link_settings_raw, 16)
            return getattr(self, '_m_link_settings', None)

        @property
        def voltage_before_dot(self):
            if hasattr(self, '_m_voltage_before_dot'):
                return self._m_voltage_before_dot

            self._m_voltage_before_dot = int(self.voltage_before_dot_raw)
            return getattr(self, '_m_voltage_before_dot', None)


    class Gprmc(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.gprmc_prefix = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.utc_time_of_position_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.status = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.latitude_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.latitude_n_or_s = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.longitude_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.longitude_e_or_w = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.ground_speed_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.course_over_ground_before_dot_raw = (self._io.read_bytes_term(46, False, True, True)).decode(u"UTF-8")
            self.course_over_ground_after_dot_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.date_str = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.magnetic_variation_before_dot_raw = (self._io.read_bytes_term(46, False, True, True)).decode(u"UTF-8")
            self.magnetic_variation_after_dot_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"UTF-8")
            self.magnetic_variation_e_or_w = (self._io.read_bytes_term(42, False, True, True)).decode(u"UTF-8")

        @property
        def utc_time_of_position_ss_after_dot(self):
            if hasattr(self, '_m_utc_time_of_position_ss_after_dot'):
                return self._m_utc_time_of_position_ss_after_dot

            self._m_utc_time_of_position_ss_after_dot = int((self.utc_time_of_position_raw)[7:9])
            return getattr(self, '_m_utc_time_of_position_ss_after_dot', None)

        @property
        def latitude_deg(self):
            if hasattr(self, '_m_latitude_deg'):
                return self._m_latitude_deg

            self._m_latitude_deg = int((self.latitude_str)[0:2])
            return getattr(self, '_m_latitude_deg', None)

        @property
        def date_mm(self):
            if hasattr(self, '_m_date_mm'):
                return self._m_date_mm

            self._m_date_mm = int((self.date_str)[2:4])
            return getattr(self, '_m_date_mm', None)

        @property
        def ground_speed_knots(self):
            if hasattr(self, '_m_ground_speed_knots'):
                return self._m_ground_speed_knots

            self._m_ground_speed_knots = int(self.ground_speed_raw)
            return getattr(self, '_m_ground_speed_knots', None)

        @property
        def magnetic_variation_after_dot(self):
            if hasattr(self, '_m_magnetic_variation_after_dot'):
                return self._m_magnetic_variation_after_dot

            self._m_magnetic_variation_after_dot = int(self.magnetic_variation_after_dot_raw)
            return getattr(self, '_m_magnetic_variation_after_dot', None)

        @property
        def utc_time_of_position_s(self):
            if hasattr(self, '_m_utc_time_of_position_s'):
                return self._m_utc_time_of_position_s

            self._m_utc_time_of_position_s = ((self.utc_time_of_position_ss_after_dot * 0.01) + self.utc_time_of_position_ss_before_dot)
            return getattr(self, '_m_utc_time_of_position_s', None)

        @property
        def date_dd(self):
            if hasattr(self, '_m_date_dd'):
                return self._m_date_dd

            self._m_date_dd = int((self.date_str)[0:2])
            return getattr(self, '_m_date_dd', None)

        @property
        def course_over_ground(self):
            if hasattr(self, '_m_course_over_ground'):
                return self._m_course_over_ground

            self._m_course_over_ground = ((self.course_over_ground_after_dot * 0.1) + self.course_over_ground_before_dot)
            return getattr(self, '_m_course_over_ground', None)

        @property
        def latitude(self):
            if hasattr(self, '_m_latitude'):
                return self._m_latitude

            self._m_latitude = ((((self.latitude_min_after_dot * 0.01) + self.latitude_min_before_dot) / 60) + self.latitude_deg)
            return getattr(self, '_m_latitude', None)

        @property
        def course_over_ground_before_dot(self):
            if hasattr(self, '_m_course_over_ground_before_dot'):
                return self._m_course_over_ground_before_dot

            self._m_course_over_ground_before_dot = int(self.course_over_ground_before_dot_raw)
            return getattr(self, '_m_course_over_ground_before_dot', None)

        @property
        def longitude(self):
            if hasattr(self, '_m_longitude'):
                return self._m_longitude

            self._m_longitude = ((((self.longitude_min_after_dot * 0.01) + self.longitude_min_before_dot) / 60) + self.longitude_deg)
            return getattr(self, '_m_longitude', None)

        @property
        def latitude_min_after_dot(self):
            if hasattr(self, '_m_latitude_min_after_dot'):
                return self._m_latitude_min_after_dot

            self._m_latitude_min_after_dot = int((self.latitude_str)[5:7])
            return getattr(self, '_m_latitude_min_after_dot', None)

        @property
        def longitude_min_before_dot(self):
            if hasattr(self, '_m_longitude_min_before_dot'):
                return self._m_longitude_min_before_dot

            self._m_longitude_min_before_dot = int((self.longitude_str)[3:5])
            return getattr(self, '_m_longitude_min_before_dot', None)

        @property
        def latitude_min_before_dot(self):
            if hasattr(self, '_m_latitude_min_before_dot'):
                return self._m_latitude_min_before_dot

            self._m_latitude_min_before_dot = int((self.latitude_str)[2:4])
            return getattr(self, '_m_latitude_min_before_dot', None)

        @property
        def longitude_deg(self):
            if hasattr(self, '_m_longitude_deg'):
                return self._m_longitude_deg

            self._m_longitude_deg = int((self.longitude_str)[0:3])
            return getattr(self, '_m_longitude_deg', None)

        @property
        def magnetic_variation(self):
            if hasattr(self, '_m_magnetic_variation'):
                return self._m_magnetic_variation

            self._m_magnetic_variation = ((self.magnetic_variation_after_dot * 0.1) + self.magnetic_variation_before_dot)
            return getattr(self, '_m_magnetic_variation', None)

        @property
        def utc_time_of_position_h(self):
            if hasattr(self, '_m_utc_time_of_position_h'):
                return self._m_utc_time_of_position_h

            self._m_utc_time_of_position_h = int((self.utc_time_of_position_raw)[0:2])
            return getattr(self, '_m_utc_time_of_position_h', None)

        @property
        def magnetic_variation_before_dot(self):
            if hasattr(self, '_m_magnetic_variation_before_dot'):
                return self._m_magnetic_variation_before_dot

            self._m_magnetic_variation_before_dot = int(self.magnetic_variation_before_dot_raw)
            return getattr(self, '_m_magnetic_variation_before_dot', None)

        @property
        def utc_time_of_position_m(self):
            if hasattr(self, '_m_utc_time_of_position_m'):
                return self._m_utc_time_of_position_m

            self._m_utc_time_of_position_m = int((self.utc_time_of_position_raw)[2:4])
            return getattr(self, '_m_utc_time_of_position_m', None)

        @property
        def course_over_ground_after_dot(self):
            if hasattr(self, '_m_course_over_ground_after_dot'):
                return self._m_course_over_ground_after_dot

            self._m_course_over_ground_after_dot = int(self.course_over_ground_after_dot_raw)
            return getattr(self, '_m_course_over_ground_after_dot', None)

        @property
        def longitude_min_after_dot(self):
            if hasattr(self, '_m_longitude_min_after_dot'):
                return self._m_longitude_min_after_dot

            self._m_longitude_min_after_dot = int((self.longitude_str)[6:8])
            return getattr(self, '_m_longitude_min_after_dot', None)

        @property
        def utc_time_of_position_ss_before_dot(self):
            if hasattr(self, '_m_utc_time_of_position_ss_before_dot'):
                return self._m_utc_time_of_position_ss_before_dot

            self._m_utc_time_of_position_ss_before_dot = int((self.utc_time_of_position_raw)[4:6])
            return getattr(self, '_m_utc_time_of_position_ss_before_dot', None)

        @property
        def date_yy(self):
            if hasattr(self, '_m_date_yy'):
                return self._m_date_yy

            self._m_date_yy = (int((self.date_str)[4:6]) + 2000)
            return getattr(self, '_m_date_yy', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Ascii85test(KaitaiStruct):
    """:field encoded: textstring.b85encstring.encoded
    :field decoded: b85string.b85decstring.decoded
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self._raw_textstring = self._io.read_bytes_term(10, False, True, True)
        _io__raw_textstring = KaitaiStream(BytesIO(self._raw_textstring))
        self.textstring = Ascii85test.B85enc(_io__raw_textstring, self, self._root)
        self._raw_b85string = self._io.read_bytes_term(10, False, True, True)
        _io__raw_b85string = KaitaiStream(BytesIO(self._raw_b85string))
        self.b85string = Ascii85test.B85dec(_io__raw_b85string, self, self._root)

    class B85enc(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_b85encstring = self._io.read_bytes_full()
            _process = satnogsdecoders.process.B85encode()
            self._raw_b85encstring = _process.decode(self._raw__raw_b85encstring)
            _io__raw_b85encstring = KaitaiStream(BytesIO(self._raw_b85encstring))
            self.b85encstring = Ascii85test.Base85string(_io__raw_b85encstring, self, self._root)


    class B85dec(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_b85decstring = self._io.read_bytes_full()
            _process = satnogsdecoders.process.B85decode()
            self._raw_b85decstring = _process.decode(self._raw__raw_b85decstring)
            _io__raw_b85decstring = KaitaiStream(BytesIO(self._raw_b85decstring))
            self.b85decstring = Ascii85test.Textstring(_io__raw_b85decstring, self, self._root)


    class Base85string(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.encoded = (self._io.read_bytes_full()).decode(u"ASCII")


    class Textstring(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.decoded = (self._io.read_bytes_full()).decode(u"ASCII")






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Netsat(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field rf_message: ax25_frame.payload.ax25_info.beacon_payload.message
    :field csp_header: ax25_frame.payload.ax25_info.csp_header
    :field compass_header_flags1: ax25_frame.payload.ax25_info.compass_header.flags1
    :field compass_header_flags2: ax25_frame.payload.ax25_info.compass_header.flags2
    :field compass_header_packet_id: ax25_frame.payload.ax25_info.compass_header.packet_id
    :field compass_header_fm_system_id: ax25_frame.payload.ax25_info.compass_header.fm_system_id
    :field compass_header_fm_subsystem_id: ax25_frame.payload.ax25_info.compass_header.fm_subsystem_id
    :field compass_header_to_system_id: ax25_frame.payload.ax25_info.compass_header.to_system_id
    :field compass_header_to_subsystem_id: ax25_frame.payload.ax25_info.compass_header.to_subsystem_id
    :field compass_header_api: ax25_frame.payload.ax25_info.compass_header.api
    :field compass_header_payload_size: ax25_frame.payload.ax25_info.compass_header.payload_size
    :field beacon_header_model: ax25_frame.payload.ax25_info.payload.beacon_header.model_type
    :field beacon_header_uid: ax25_frame.payload.ax25_info.payload.beacon_header.uid
    :field beacon_header_type: ax25_frame.payload.ax25_info.payload.beacon_header.type
    :field beacon_header_length: ax25_frame.payload.ax25_info.payload.beacon_header.length
    :field beacon_header_timestamp: ax25_frame.payload.ax25_info.payload.beacon_header.timestamp_raw.timestamp
    :field beacon_payload_beacon_rate: ax25_frame.payload.ax25_info.payload.beacon_payload.beacon_rate
    :field beacon_payload_uptime: ax25_frame.payload.ax25_info.payload.beacon_payload.uptime
    :field beacon_payload_status_aocs_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.aocs_active
    :field beacon_payload_status_radio2_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.radio2_active
    :field beacon_payload_status_powerpath_b_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.powerpath_b_active
    :field beacon_payload_status_powerpath_a_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.powerpath_a_active
    :field beacon_payload_status_mcub_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.is_mcu_b_active
    :field beacon_payload_status_panel_neg_x_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.panel_x_neg_active
    :field beacon_payload_status_panel_pos_x_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.panel_x_pos_active
    :field beacon_payload_status_panel_neg_y_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.panel_y_neg_active
    :field beacon_payload_status_panel_pos_y_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.panel_y_pos_active
    :field beacon_payload_status_panel_neg_z_active: ax25_frame.payload.ax25_info.payload.beacon_payload.status_flags.panel_z_neg_active
    :field beacon_payload_powerpath_a_temp: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_a_temp_soc.temp
    :field beacon_payload_powerpath_a_state_of_charge: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_a_temp_soc.soc
    :field beacon_payload_powerpath_b_temp: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_b_temp_soc.temp
    :field beacon_payload_powerpath_b_state_of_charge: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_b_temp_soc.soc
    :field beacon_payload_powerpath_a_voltage: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_a_iup.u
    :field beacon_payload_powerpath_a_current: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_a_iup.i
    :field beacon_payload_powerpath_a_power: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_a_iup.p
    :field beacon_payload_powerpath_b_voltage: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_b_iup.u
    :field beacon_payload_powerpath_b_current: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_b_iup.i
    :field beacon_payload_powerpath_b_power: ax25_frame.payload.ax25_info.payload.beacon_payload.powerpath_b_iup.p
    :field beacon_payload_power_consumption: ax25_frame.payload.ax25_info.payload.beacon_payload.power_total
    :field beacon_payload_obc_temp: ax25_frame.payload.ax25_info.payload.beacon_payload.temperature_obc
    :field beacon_payload_panel_neg_x_temp: ax25_frame.payload.ax25_info.payload.beacon_payload.temperature_panel.xneg
    :field beacon_payload_panel_pos_x_temp: ax25_frame.payload.ax25_info.payload.beacon_payload.temperature_panel.xpos
    :field beacon_payload_panel_neg_y_temp: ax25_frame.payload.ax25_info.payload.beacon_payload.temperature_panel.yneg
    :field beacon_payload_panel_pos_y_temp: ax25_frame.payload.ax25_info.payload.beacon_payload.temperature_panel.ypos
    :field beacon_payload_panel_neg_z_temp: ax25_frame.payload.ax25_info.payload.beacon_payload.temperature_panel.zneg
    :field beacon_payload_seu_ram: ax25_frame.payload.ax25_info.payload.beacon_payload.seu_ram
    :field beacon_payload_seu_rom: ax25_frame.payload.ax25_info.payload.beacon_payload.seu_rom
    :field beacon_payload_freq: ax25_frame.payload.ax25_info.payload.beacon_payload.frequency_radio
    :field beacon_payload_misc: ax25_frame.payload.ax25_info.payload.compass_misc
    :field beacon_payload_crc: ax25_frame.payload.ax25_info.payload.frame_crc32c
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Netsat.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Netsat.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Netsat.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Netsat.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Netsat.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Netsat.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Netsat.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Netsat.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Netsat.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Netsat.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Netsat.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Netsat.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class TemperaturePanel(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.xpos = self._io.read_s1()
            self.xneg = self._io.read_s1()
            self.ypos = self._io.read_s1()
            self.yneg = self._io.read_s1()
            self.zneg = self._io.read_s1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self._parent.ax25_header.src_callsign_raw.callsign_ror.is_netsat
            if _on == True:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Netsat.NetsatFrame(_io__raw_ax25_info, self, self._root)
            else:
                self.ax25_info = self._io.read_bytes_full()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"utf-8")

        @property
        def is_netsat(self):
            if hasattr(self, '_m_is_netsat'):
                return self._m_is_netsat

            self._m_is_netsat =  ((self.callsign == u"DP2NSA") or (self.callsign == u"DP2NSB") or (self.callsign == u"DP2NSC") or (self.callsign == u"DP2NSD")) 
            return getattr(self, '_m_is_netsat', None)


    class NetsatFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.csp_header = self._io.read_u4le()
            self._raw_compass_header = self._io.read_bytes(10)
            _io__raw_compass_header = KaitaiStream(BytesIO(self._raw_compass_header))
            self.compass_header = Netsat.CompassHeader(_io__raw_compass_header, self, self._root)
            _on = self.compass_header.api
            if _on == 14:
                self._raw_payload = self._io.read_bytes(self.compass_header.payload_size)
                _io__raw_payload = KaitaiStream(BytesIO(self._raw_payload))
                self.payload = Netsat.NetsatBeacon(_io__raw_payload, self, self._root)
            elif _on == 103:
                self._raw_payload = self._io.read_bytes(self.compass_header.payload_size)
                _io__raw_payload = KaitaiStream(BytesIO(self._raw_payload))
                self.payload = Netsat.NetsatRfMessage(_io__raw_payload, self, self._root)
            else:
                self.payload = self._io.read_bytes(self.compass_header.payload_size)
            self.compass_misc = self._io.read_u2le()
            self.frame_crc32c = self._io.read_u4le()


    class BeaconHeaderTimestamp(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_header_ts = []
            for i in range(6):
                self.beacon_header_ts.append(self._io.read_u1())


        @property
        def timestamp_lsb(self):
            if hasattr(self, '_m_timestamp_lsb'):
                return self._m_timestamp_lsb

            self._m_timestamp_lsb = (((self.beacon_header_ts[0] + (self.beacon_header_ts[1] << 8)) + (self.beacon_header_ts[2] << 16)) + (self.beacon_header_ts[3] << 24))
            return getattr(self, '_m_timestamp_lsb', None)

        @property
        def timestamp_msb(self):
            if hasattr(self, '_m_timestamp_msb'):
                return self._m_timestamp_msb

            self._m_timestamp_msb = ((self.beacon_header_ts[4] + (self.beacon_header_ts[5] << 8)) + 1)
            return getattr(self, '_m_timestamp_msb', None)

        @property
        def timestamp(self):
            """padded unix timestamp (64bit) in ms."""
            if hasattr(self, '_m_timestamp'):
                return self._m_timestamp

            self._m_timestamp = (((self.timestamp_msb * 4294967296) + self.timestamp_lsb) - 4294967296)
            return getattr(self, '_m_timestamp', None)


    class PpIUP(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.u = self._io.read_s2le()
            self.i = self._io.read_s2le()
            self.p = self._io.read_s2le()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class NetsatRfMessage(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.offset_0 = []
            for i in range(6):
                self.offset_0.append(self._io.read_u1())

            self.message = (self._io.read_bytes((self._parent.compass_header.payload_size - 6))).decode(u"utf-8")
            self.rf_message_crc = self._io.read_u2le()


    class NetsatBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_header = Netsat.BeaconHeader(self._io, self, self._root)
            _on = self.beacon_header.length
            if _on == 42:
                self.beacon_payload = Netsat.TelemetryPayload(self._io, self, self._root)


    class CompassHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.flags1 = self._io.read_u1()
            self.flags2 = self._io.read_u1()
            self.packet_id = self._io.read_u2le()
            self.fm_system_id = self._io.read_u1()
            self.fm_subsystem_id = self._io.read_u1()
            self.to_system_id = self._io.read_u1()
            self.to_subsystem_id = self._io.read_u1()
            self.api = self._io.read_u1()
            self.payload_size = self._io.read_u1()


    class BeaconHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.model_type = self._io.read_u1()
            self.uid = self._io.read_u2le()
            self.type = self._io.read_u1()
            self.length = self._io.read_u1()
            self._raw_timestamp_raw = self._io.read_bytes(6)
            _io__raw_timestamp_raw = KaitaiStream(BytesIO(self._raw_timestamp_raw))
            self.timestamp_raw = Netsat.BeaconHeaderTimestamp(_io__raw_timestamp_raw, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Netsat.Callsign(_io__raw_callsign_ror, self, self._root)


    class TelemetryPayload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_rate = self._io.read_u4le()
            self.uptime = self._io.read_u4le()
            self._raw_status_flags = self._io.read_bytes(2)
            _io__raw_status_flags = KaitaiStream(BytesIO(self._raw_status_flags))
            self.status_flags = Netsat.BeaconStatusFlags(_io__raw_status_flags, self, self._root)
            self.powerpath_a_temp_soc = Netsat.PpTempsoc(self._io, self, self._root)
            self.powerpath_b_temp_soc = Netsat.PpTempsoc(self._io, self, self._root)
            self.powerpath_a_iup = Netsat.PpIUP(self._io, self, self._root)
            self.powerpath_b_iup = Netsat.PpIUP(self._io, self, self._root)
            self.power_total = self._io.read_s4le()
            self.temperature_obc = self._io.read_s1()
            self.temperature_panel = Netsat.TemperaturePanel(self._io, self, self._root)
            self.frequency_radio = self._io.read_u4le()
            self.seu_rom = self._io.read_u1()
            self.seu_ram = self._io.read_u1()


    class BeaconStatusFlags(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.blank_012 = self._io.read_bits_int_be(3)
            self.aocs_active = self._io.read_bits_int_be(1) != 0
            self.radio_2_active = self._io.read_bits_int_be(1) != 0
            self.powerpath_b_active = self._io.read_bits_int_be(1) != 0
            self.powerpath_a_active = self._io.read_bits_int_be(1) != 0
            self.is_mcu_b_active = self._io.read_bits_int_be(1) != 0
            self.blank_mode = self._io.read_bits_int_be(2)
            self.panel_xneg_active = self._io.read_bits_int_be(1) != 0
            self.panel_xpos_active = self._io.read_bits_int_be(1) != 0
            self.panel_yneg_active = self._io.read_bits_int_be(1) != 0
            self.panel_ypos_active = self._io.read_bits_int_be(1) != 0
            self.panel_zneg_active = self._io.read_bits_int_be(1) != 0
            self.blank_15 = self._io.read_bits_int_be(1) != 0


    class PpTempsoc(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.temp = self._io.read_s1()
            self.soc = self._io.read_s1()



